{
  Drake Rider

  (C) 2002 Vampyre SoftWorks
  Created by :  Marek Mauder  
}	




unit D3DClass;

interface

uses Windows, Forms, Classes, SysUtils, DXStuff,
  D3DX8, INIFiles, DirectXGraphics, gameTypes, Messages,
  D3DFunc, Errors, Graphics, WinFunc,DXObj,MainGame;

const
  // for SetDefaults
  SD_CREATE = 1;
  SD_AFTERINIT = 2;
  SD_RESET = 3;
  SD_PREINIT = 4;


type

  TGameOpt = record
    ShowStats: boolean;
    ClearDev: boolean;
    ClearColor: dword;
    TexFilter: dword;
    IntWidth: long;
    IntHeight: long;
    PlayMus: boolean;
    PlaySfx: boolean;
    SfxVol: long;
    MusVol: long;
    RandPly: boolean;
    GameSpeed:long;
  end;


  TD3DApp = class
  private
    FState: long;
  protected

    procedure SetState(Value: long); virtual;

  public


    Window: TDXForm;
    GameOpt: TGameOpt;


    Device: IDevice;
    D3D: ID3D;
    DeskMode: TD3DDisplayMode;

    ViewMatrix: TD3DMatrix;
    WorldMatrix: TD3DMatrix;
    ProjectMatrix: TD3DMatrix;

    Timer: TDXTimer;

    StatsFont: TFont;
    StatsFontDX: ID3DXFont;
    Sprite: ID3DXSprite;

    INI: TINIFile;

    constructor Create(AWindow: TDXForm); virtual;
    destructor Destroy; override;

    procedure MsgHandler(var Msg: TMsg; var Handled: boolean); virtual;
    procedure InitD3D; virtual;
    procedure SetDefaults(setdef: long); virtual;

    procedure SetD3DPP; virtual;
    procedure Draw; virtual;
    procedure Render; virtual;
    procedure OnTimer(Sender: TObject; Lag: Integer);
    procedure DoTimer; virtual;

    procedure DrawStats; virtual;

    procedure Reset; virtual;
    procedure RestoreObj; virtual;
    procedure DeleteObj; virtual;

    procedure ToggleFullScreen; virtual;
    procedure SetFullParams; virtual;
    procedure SetRes(width, height, bits: long; full: boolean = false; reset: boolean = false); virtual;

    procedure ExitGame; virtual;

    procedure ReadCfg; virtual;
    procedure WriteCfg; virtual;


    property State: long read FState write SetState;


  end;

implementation



constructor TD3DApp.Create(AWindow: TDXForm);
begin
  inherited Create;
  Window := AWindow;
  Application.OnMessage := MsgHandler;
  Timer := TDXTimer.Create(Window);
  Timer.OnTimer := OnTimer;

  Window.StoreWindow;


  SetDefaults(SD_CREATE);
  ReadCfg;

  SetDefaults(SD_PREINIT);

  D3D := CreateD3D;
  if D3D = nil then ShowError(ERR_D3DNOTCREATED);
  InitD3D;
  Timer.Initialize;
  SetDefaults(SD_AFTERINIT);


  State := 0;

end;


destructor TD3DApp.Destroy;
begin

  inherited Destroy;
end;

procedure TD3DApp.ExitGame;
begin
  DeleteObj;
  Timer.Free;
  Device := nil;
  D3D := nil;
  
  WriteCfg;

  Destroy;
end;

procedure TD3DApp.ReadCfg;
begin
  INI := TINIFile.Create(ExeDir + 'Data\Config\config.ini');
  GfxOpt.Width:=INI.ReadInteger('Gfx','Width',640);
  GfxOpt.Height:=INI.ReadInteger('Gfx','Height',480);
  GfxOpt.Bits:=INI.ReadInteger('Gfx','Bits',16);
  GfxOpt.BackBuffers:=INI.ReadInteger('Gfx','Buffers',1);
  GfxOpt.FullScreen:=INI.ReadBool('Gfx','FullScr',false);
  GfxOpt.DepthBuffer:=INI.ReadBool('Gfx','Depth',false);
  GfxOpt.WaitVBlank:=INI.ReadBool('Gfx','WaitVBlank',false);
  GfxOpt.Hal:=INI.ReadBool('Gfx','Hal',true);

  GameOpt.TexFilter:=INI.ReadInteger('Game','Filter',2);
  GameOpt.IntWidth:=INI.ReadInteger('Game','iW',800);
  GameOpt.IntHeight:=INI.ReadInteger('Game','iH',600);
  GameOpt.SfxVol:=INI.ReadInteger('Game','SfxVol',5);
  GameOpt.MusVol:=INI.ReadInteger('Game','MusVol',3);
  GameOpt.ShowStats:=INI.ReadBool('Game','ShowStats',true);
  GameOpt.ClearDev:=INI.ReadBool('Game','Clear',true);
  GameOpt.PlayMus:=INI.ReadBool('Game','PlayMUS',true);
  GameOpt.PlaySfx:=INI.ReadBool('Game','PlaySFX',true);
  GameOpt.RandPly:=INI.ReadBool('Game','RandPly',false);
  GameOpt.GameSpeed:=INI.ReadInteger('Game','GameSpeed',3);

  if Debug then GfxOpt.FullScreen:=false;

end;


procedure TD3DApp.WriteCfg;
begin

  INI.WriteInteger('Gfx','Width',GfxOpt.Width);
  INI.WriteInteger('Gfx','Height',GfxOpt.Height);
  INI.WriteInteger('Gfx','Bits',GfxOpt.Bits);
  INI.WriteInteger('Gfx','Buffers',GfxOpt.BackBuffers);
  INI.WriteBool('Gfx','FullScr',GfxOpt.FullScreen);
  INI.WriteBool('Gfx','Depth',GfxOpt.DepthBuffer);
  INI.WriteBool('Gfx','WaitVBlank',GfxOpt.WaitVBlank);
  INI.WriteBool('Gfx','Hal',GfxOpt.Hal);

  INI.WriteInteger('Game','Filter',GameOpt.TexFilter);
  INI.WriteInteger('Game','iW',GameOpt.IntWidth);
  INI.WriteInteger('Game','iH',GameOpt.IntHeight);
  INI.WriteInteger('Game','SfxVol',GameOpt.SfxVol);
  INI.WriteInteger('Game','MusVol',GameOpt.MusVol);
  INI.WriteBool('Game','ShowStats',GameOpt.ShowStats);
  INI.WriteBool('Game','Clear',GameOpt.ClearDev);
  INI.WriteBool('Game','PlayMUS',GameOpt.PlayMus);
  INI.WriteBool('Game','PlaySFX',GameOpt.PlaySfx);
  INI.WriteBool('Game','RandPly',GameOpt.RandPly);
  INI.WriteInteger('Game','GameSpeed',GameOpt.GameSpeed);


  INI.Free;
end;



procedure TD3DApp.SetDefaults(setdef: long);
begin


  case SetDef of
    SD_CREATE:
      begin
        GfxOpt.Width := 640;
        GfxOpt.Height := 480;
        GfxOpt.Bits := 16;
        GfxOpt.FullScreen := false;
        GfxOpt.WaitVBlank := not true;
        GfxOpt.Hal := true;
        GfxOpt.BackBuffers := 1;
        GfxOpt.Handle := 0;//Window.Handle;
        GfxOpt.DepthBuffer := false;

        Timer.ActiveOnly := true;
        Timer.Enabled := true;
        Timer.Interval := 0;

        GameOpt.ShowStats := true;
        GameOpt.ClearDev := true;
        GameOpt.ClearColor := $FF000000;
        GameOpt.TexFilter := D3DTEXF_LINEAR;
        GameOpt.IntWidth := 800;
        GameOpt.IntHeight := 600;
        GameOpt.PlayMus := false;


        StatsFont := TFont.Create;
        StatsFont.Name := 'Arial';
        StatsFont.Size := 10;
        StatsFont.Style := [fsBold];

        Fillchar(d3dpp, sizeof(d3dpp), 0);

        Window.BorderStyle := bsNone;

      end;
    SD_AFTERINIT:
      begin
       //must create cursor



      end;

  end;

end;

procedure TD3DApp.MsgHandler(var Msg: TMsg; var Handled: boolean);
begin

  case Msg.message of
    WM_KEYDOWN:

  end;

end;




procedure TD3DApp.SetState(Value: long);
begin
  FState := Value;
end;



procedure TD3DApp.DrawStats;
begin
  with StatsFontDX do
  begin

    _R := Rect(10, 10, 100, 30);
    DrawTextA(PChar(Format('FPS: %d', [Timer.Framerate])), -1, _R, 0,
      D3DCOLOR_ARGB(200, 255, 255, 50));



  end;
end;

procedure TD3DApp.Draw;
var
  Hr: HResult;
begin


  Hr := Device.TestCooperativeLevel;
  if (hr = D3DERR_DEVICELOST) then Exit;
  if (hr = D3DERR_DEVICENOTRESET) then Reset;


  if GameOpt.ClearDev then Device.Clear(0, nil, D3DCLEAR_TARGET, GameOpt.ClearColor, 1, 0);
  Device.BeginScene;




  Render;

  if GameOpt.ShowStats then DrawStats;

  Cursor.Update;

  Device.EndScene;

//  if not tWin(Window).Console.Visible then
 Device.Present(nil, nil,0 , nil)
 // else ;///Device.Present(nil, nil,tWin(Window).View.Handle , nil);

{  if tWin(Window).Console.Visible then }//tWin(Window).Console.invalidate;//PaintTo(GetDC(Window.Handle),10,10);

  //else  Device.Present(nil, nil,tWin(Window).Console.Handle , nil)
end;

procedure TD3DApp.Render;
begin
end;

procedure TD3DApp.InitD3D;
var
  old: bool;
  i, dtype: long;
  nmodes: long;
  DModes: array of TD3DDisplayMode;
begin
  GfxOpt._640x480rate := 0;
  GfxOpt._800x600rate := 0;
  GfxOpt._1024x768rate := 0;

  D3D.GetAdapterDisplayMode(D3DADAPTER_DEFAULT, deskmode);
  nmodes := D3D.GetAdapterModeCount(D3DADAPTER_DEFAULT);
  Setlength(DModes, nmodes);
  for i := 0 to nmodes - 1 do
  begin
    D3D.EnumAdapterModes(D3DADAPTER_DEFAULT, i, DModes[i]);
    if (DModes[i].Width = 640) and (DModes[i].height = 480)
      and (DModes[i].RefreshRate > GfxOpt._640x480rate) then GfxOpt._640x480rate := DModes[i].RefreshRate;

    if (DModes[i].Width = 800) and (DModes[i].height = 600)
      and (DModes[i].RefreshRate > GfxOpt._800x600rate)
      and (DModes[i].RefreshRate <> 75) then GfxOpt._800x600rate := DModes[i].RefreshRate;

    if (DModes[i].Width = 1024) and (DModes[i].height = 768)
      and (DModes[i].RefreshRate > GfxOpt._1024x768rate) then GfxOpt._1024x768rate := DModes[i].RefreshRate;

  end;

  old := gfxopt.FullScreen;
  gfxopt.FullScreen := false;

  SetFullParams;
  SetD3Dpp;
  if GFXOpt.Hal then
    dtype := D3DDEVTYPE_HAL
  else
    dtype := D3DDEVTYPE_REF;
  if Failed(D3D.CreateDevice(D3DADAPTER_DEFAULT, dtype,Window.Handle,
    D3DCREATE_SOFTWARE_VERTEXPROCESSING,
    d3dpp, Device)) then ShowError(ERR_DEVINITFAIL); ;

  gfxopt.FullScreen := old;
  Reset;


end;





procedure TD3DApp.RestoreObj;
begin

  StatsFontDX := nil;
  if Failed(D3DXCreateFont(Device, StatsFont.Handle, StatsFontDX)) then ShowError(ERR_CREATEOBJ);

  Sprite := nil;
  if Failed(D3DXCreateSprite(Device, Sprite)) then ShowError(ERR_CREATEOBJ);


  Device.SetRenderState(D3DRS_ALPHABLENDENABLE, ord(TRUE));
  Device.SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
  Device.SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
  Device.SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_MODULATE);
  Device.SetTextureStageState(0, D3DTSS_MINFILTER, GameOpt.TexFilter);
  Device.SetTextureStageState(0, D3DTSS_MAGFILTER, GameOpt.TexFilter);
  Device.SetRenderState(D3DRS_LIGHTING, ord(TRUE));
  Device.SetRenderState(D3DRS_AMBIENT, $FFFFFFFF);
  Device.SetRenderState(D3DRS_ZENABLE, ord(not TRUE));

  Device.SetTransform(D3DTS_PROJECTION, ProjectMatrix);
  Device.SetTransform(D3DTS_WORLD, WorldMatrix);
  Device.SetTransform(D3DTS_VIEW, ViewMatrix);

  if GfxOpt.FullScreen then
   begin
     Window.ClientWidth:=GfxOpt.Width;
     Window.ClientHeight:=GfxOpt.Height;

   end;


end;

procedure TD3DApp.DeleteObj;
begin
//  StatsFont := nil;
//  Sprite := nil;


end;



procedure TD3DApp.OnTimer(Sender: TObject; Lag: Integer);
begin
  DoTimer;
  Draw;
end;

procedure TD3DApp.Reset;
begin
  SetFullParams;
  SetD3DPP;
  DeleteObj;
  if Failed(Device.Reset(D3DPP)) then ShowError(ERR_DEVRESET);
  RestoreObj;

end;

procedure TD3DApp.ToggleFullScreen;
begin
  GfxOpt.FullScreen := not GfxOpt.FullScreen;
  Reset;
end;

procedure TD3DApp.SetFullParams;
begin
  if GfxOpt.FullScreen then
  begin
    if d3dpp.Windowed then
    begin
      Window.StoreWindow;
      Window.ClientWidth:=GfxOpt.Width;
      Window.ClientHeight:=GfxOpt.Height;

      Window.BorderStyle := bsNone;
    end;
  end
  else if not d3dpp.Windowed then
  begin
    Window.ReStoreWindow;
    Window.BorderStyle := bsSingle;
    Window.Left := (DeskMode.Width - Window.Width) div 2;
    Window.Top := (DeskMode.Height - Window.height) div 2;


  end

end;

procedure TD3DApp.SetD3DPP;
var
  d3ddm: TD3DDisplayMode;
begin
  Fillchar(d3dpp, sizeof(d3dpp), 0);
  if GfxOpt.FullScreen then
  begin
    d3dpp.BackBufferWidth := GfxOpt.Width;
    d3dpp.BackBufferHeight := GfxOpt.Height;
    case GfxOpt.Bits of
      16: d3dpp.BackBufferFormat := D3DFMT_R5G6B5;
      32: d3dpp.BackBufferFormat := D3DFMT_X8R8G8B8;
    else
      d3dpp.BackBufferFormat := D3DFMT_R5G6B5;
    end;
    if not GfxOpt.WaitVBlank then
      d3dpp.FullScreen_PresentationInterval := D3DPRESENT_INTERVAL_IMMEDIATE
    else
      d3dpp.FullScreen_PresentationInterval := 0;

    case GfxOpt.Width of
      640: d3dpp.FullScreen_RefreshRateInHz := GfxOpt._640x480rate;
      800: d3dpp.FullScreen_RefreshRateInHz := GfxOpt._800x600rate;
      1024: d3dpp.FullScreen_RefreshRateInHz := GfxOpt._1024x768rate;
    else
      d3dpp.FullScreen_RefreshRateInHz := 0;
    end;


  end
  else
  begin
    d3dpp.BackBufferWidth := 0;
    d3dpp.BackBufferHeight := 0;
    d3dpp.BackBufferFormat := deskmode.Format;
  end;

  d3dpp.SwapEffect := D3DSWAPEFFECT_DISCARD;
  d3dpp.BackBufferCount := GfxOpt.BackBuffers;
  d3dpp.hDeviceWindow := Window.Handle;
  d3dpp.Windowed := not GfxOpt.FullScreen;
  if GfxOpt.DepthBuffer then
  begin
    d3dpp.EnableAutoDepthStencil := true;
    d3dpp.AutoDepthStencilFormat := D3DFMT_D16;
  end;

end;


procedure TD3DApp.SetRes(width, height, bits: long; full: boolean; reset: boolean);
begin
  GfxOpt.Width := width;
  GfxOpt.Height := height;
  GfxOpt.Bits := bits;
  GfxOpt.FullScreen := full;


  if Reset then Self.Reset;

end;

procedure TD3DApp.DoTimer;
begin


end;




end.

