{
  Drake Rider

  (C) 2002 Vampyre SoftWorks
  Created by :  Marek Mauder  
}	


unit DXObj;

interface

uses Windows, SysUtils, Classes ,DirectXGraphics,
  D3DX8, gameTypes, Errors, MainGame, WinFunc,D3DFunc;


type

  TPanelVertex = record
    x, y, z: single;
    color: dword;
    u, v: single;
  end;

  TPVArray = array[0..3] of TPanelVertex;

const
  D3DFVF_PANELVERTEX = D3DFVF_XYZ or D3DFVF_DIFFUSE or D3DFVF_TEX1;

  MIN_U=0.01;
  MAX_U=0.99;
  MIN_V=0.01;
  MAX_V=0.99;
  

  
type


  TDXTexture = class
    Name: string;
    Tex: ITexture;
    Device: IDevice;
    Filter: boolean;
    Colorkey: dword;

    Tag: long;
    constructor Create(ADev: IDevice; AName: string; afilter: boolean = true; acolorkey: dword = 0; load: boolean = false); virtual;
    destructor Destroy; override;
    procedure Load; virtual;
    procedure Change(AName: string; afilter: boolean = true; acolorkey: dword = 0; load: boolean = false); virtual;

  end;


  TDXObj = class(TPersistent)
    public
    Device: IDevice;
    Mtrl: TMtrl;
    Texture: TDXTexture;

    PosMatrix,
      RotMatrix,
      Scalematrix: TD3DMatrix;

    VBuffer: IVBuffer;
    Visible: boolean;
      Name: string;

    Tag1,
      Tag2,
      Tag3: long;
  

    constructor Create(ADev: IDevice); virtual;
    destructor Destroy; override;
    procedure Render; virtual;
    procedure Init; virtual; abstract;

    procedure Hide; virtual;
    procedure Show; virtual;
    procedure ToggleVisible; virtual;

    procedure RotateAbs3D(x, y, z: single); virtual;
    procedure RotateRel3D(x, y, z: single); virtual;
    procedure Move3D(x, y, z: single); virtual;

    procedure RotateAbs2D(angle: single); virtual;
    procedure RotateRel2D(angle: single); virtual;
    procedure Move2D(x, y: single); virtual;

    procedure ScaleAbs3D(x, y, z: single); virtual;
    procedure ScaleRel3D(x, y, z: single); virtual;

    procedure ScaleAbs2D(x, y: single); virtual;
    procedure ScaleRel2D(x, y: single); virtual;

  end;

  TDXBlend = (bl_normal, bl_add);

  TDXPanel = class(TDXObj)

  public
    Verts: TPVArray;
    Color: TD3DColor;
    Bounds: TRect;
    Blend: TDXBlend;

    Z:long;
    constructor Create(ADev: IDevice); overload; override;
    constructor Create(ADev: IDevice;
      ABounds: TRect; AColor: TD3DColor; Tex: TDXTexture = nil;
      TexName: string = ''; AInit: boolean = true;
      ITex: ITexture = nil); reintroduce; overload; virtual;


    destructor Destroy; override;

    procedure Init; override;
    procedure Render; override;
    procedure SetPos(x, y: long); virtual;
    function GetPos: TPoint; virtual;
    function GetRealPos:TVector2;virtual;

    procedure Resize(w, h: long); virtual;
    procedure Center(x, y: boolean); virtual;

    procedure RotateAbs(angle: single); virtual;
    procedure RotateRel(angle: single); virtual;
    procedure ScaleAbs(x, y: single); virtual;
    procedure ScaleRel(x, y: single); virtual;
    procedure Move(x, y: single); virtual;

    procedure SetColor(col: TD3DColor); virtual;

    procedure MouseStart; virtual; abstract;
    procedure MouseStop; virtual; abstract;

    procedure SetUV(minu, minv, maxu, maxv: single; Ainit: boolean = true);


  end;


  TDXPanelEx=class(TDXPanel)
            public
             Frames:long;
             AFrame:long;
             FWidth:long;
             FHeight:long;
             Rate:long;
             CanAnim:boolean;
             step_u,step_v:single;

             procedure NextFrame;virtual;
             procedure SetAnim(w,h,r:long;can:boolean=true);virtual;
             procedure DoTimer;virtual;
             procedure SetPosUV(minu, minv, maxu, maxv: single);virtual;
               
             end;
  

var
  DefMtrl: TMtrl;

implementation

uses DXStuff;

///////////////// TDXTEXTURE

constructor TDXTEXTURE.Create(ADev: IDevice; AName: string; afilter: boolean; acolorkey: dword; load: boolean);
begin
  inherited Create;
  Device := ADev;
  Change(AName, AFilter, AColorkey, Load);
end;

destructor TDXTEXTURE.Destroy;
begin
  Tex := nil;
  inherited;
end;

procedure TDXTexture.Change(AName: string; afilter: boolean = true; acolorkey: dword = 0; load: boolean = false);
begin
  Name := AName;
  Filter := AFilter;
  ColorKey := AColorkey;
  if Load then Self.Load;
end;


procedure TDXTEXTURE.Load;
var
  Filt: dword;
begin
  if Name = '' then Exit;
  Tex := nil;

  if Filter then
    Filt := D3DX_FILTER_LINEAR
  else
    Filt := D3DX_FILTER_NONE;

  Tex := LoadTexture(Device, Name, Filt, Colorkey);
  if Tex = nil then ShowError(ERR_TEXLOAD);


end;


/////////////////////////// TDXOBJ

constructor TDXObj.Create(ADev: IDevice);
begin
  inherited Create;
  Device := ADev;
  Texture := TDXTexture.Create(Device, '');
  Mtrl := DefMtrl;
  Visible := true;
  PosMatrix := Identmatrix;
  RotMatrix := Identmatrix;
  ScaleMatrix := Identmatrix;
  Name:='DXObj';
  Tag1:=0;
  Tag2:=0;
  tag3:=0;
  
end;


destructor TDXObj.Destroy;
begin

  VBuffer := nil;
//  Texture.Free;
  Device := nil;
  inherited;
end;

procedure TDXObj.Render;
var
  Transform: TD3DMatrix;
begin
  Device.SetMaterial(Mtrl);
  Device.SetTexture(0, Texture.Tex);

  D3DXMatrixMultiply(Transform, ScaleMatrix, RotMatrix);
  D3DXMatrixMultiply(Transform, Transform, Posmatrix);
  Device.SetTransform(D3DTS_WORLD, Transform);


end;

procedure TDXObj.Hide;
begin
  Visible := false;
end;


procedure TDXObj.Show;
begin
  Visible := true;
end;

procedure TDXObj.ToggleVisible;
begin
  Visible := not Visible;
end;

procedure TDXObj.RotateAbs3D(x, y, z: single);
begin
  D3DXMatrixRotationYawPitchRoll(Rotmatrix, degtorad(y), degtorad(x), degtorad(z));
end;

procedure TDXObj.RotateRel3D(x, y, z: single);
var
  Temp: TD3DMatrix;
begin
  D3DXMatrixRotationYawPitchRoll(Temp, degtorad(y), degtorad(x), degtorad(z));
  D3DXMatrixMultiply(Rotmatrix, Rotmatrix, Temp);

end;


procedure TDXObj.Move3D(x, y, z: single);
var
  Temp: TD3DMatrix;
begin
  D3DXMatrixTranslation(Temp, X, Y, Z);
  D3DXMatrixMultiply(Posmatrix, Posmatrix, Temp);

end;


procedure TDXObj.RotateAbs2D(angle: single);
begin
  D3DXMatrixRotationZ(RotMatrix, degtorad(angle));
end;

procedure TDXObj.RotateRel2D(angle: single);
var
  Temp: TD3DMatrix;
begin
  D3DXMatrixRotationZ(Temp, degtorad(angle));
  D3DXMatrixMultiply(Rotmatrix, Rotmatrix, Temp);
end;

procedure TDXObj.Move2D(x, y: single);
var
  Temp: TD3DMatrix;
begin
  D3DXMatrixTranslation(Temp, X, Y, 0);
  D3DXMatrixMultiply(Posmatrix, Posmatrix, Temp);
end;

procedure TDXObj.ScaleAbs3D(x, y, z: single);
begin
  D3DXMatrixScaling(ScaleMatrix, x, y, z);
end;

procedure TDXObj.ScaleRel3D(x, y, z: single);
var
  Temp: TD3DMatrix;
begin
  D3DXMatrixScaling(Temp, X, Y, Z);
  D3DXMatrixMultiply(Scalematrix, Scalematrix, Temp);
end;

procedure TDXObj.ScaleAbs2D(x, y: single);
begin
  D3DXMatrixScaling(ScaleMatrix, x, y, 1);
end;


procedure TDXObj.ScaleRel2D(x, y: single);
var
  Temp: TD3DMatrix;
begin
  D3DXMatrixScaling(Temp, X, Y, 1);
  D3DXMatrixMultiply(Scalematrix, Scalematrix, Temp);
end;





/////////////////////// TDXPANEL

constructor TDXPanel.Create(ADev: IDevice);
var
  i: long;

begin
  inherited;
  Color := $FFFFFFFF;
  Bounds := Rect(10, 10, 200, 40);
  for i := 0 to 3 do
    Verts[i].color := Color;



end;

constructor TDXPanel.Create(ADev: IDevice;
  ABounds: TRect; AColor: TD3DColor; Tex: TDXTexture;
  TexName: string; AInit: boolean;
  ITex: ITexture);
var
  i: long;
begin
  inherited Create(ADev);
  Name:='DXPanel';
  Blend := bl_normal;
  Color := AColor;
  Bounds := ABounds;
  for i := 0 to 3 do
    Verts[i].color := Color;
  if Tex <> nil then
  begin
    Texture.Free;
    Texture := Tex;

  end
  else
  begin

    if Texname <> '' then
      Texture.Change(TexName, Texture.Filter, Texture.ColorKey, True);
    if ITex <> nil then
    begin
      Texture.Name := '??tex??';
      Texture.Tex := nil;
      Texture.Tex := ITex;
    end;
  end;

  SetUV(MIN_U, MIN_V, MAX_U, MAX_V, AInit);

  SetPos(Bounds.Left, Bounds.Top);

end;

destructor TDXPanel.Destroy;
begin
  if TexMgr.Find(Texture.Name) = Texture
    then
    Texture := nil
  else
    Texture.Free;

  inherited;
end;


procedure TDXPanel.SetUV(minu, minv, maxu, maxv: single; Ainit: boolean = true);
begin
  Verts[0].u := minu; Verts[0].v := minv;
  Verts[1].u := minu; Verts[1].v := maxv;
  Verts[2].u := maxu; Verts[2].v := minv;
  Verts[3].u := maxu; Verts[3].v := maxv;


  if Ainit then Init;

  end;

procedure TDXPanel.Init;
var
  pb: PByte;
begin

  Verts[0].x := -Bounds.Right div 2;
  Verts[1].x := -Bounds.Right div 2;
  Verts[2].x := Bounds.Right div 2;
  Verts[3].x := Bounds.Right div 2;

  Verts[0].y := -Bounds.Bottom div 2;
  Verts[1].y := Bounds.Bottom div 2;
  Verts[2].y := -Bounds.Bottom div 2;
  Verts[3].y := Bounds.Bottom div 2;


  Verts[0].z := 0;
  Verts[1].z := 0;
  Verts[2].z := 0;
  Verts[3].z := 0;


  VBuffer := nil;

  Device.CreateVertexBuffer(SizeOf(TPVArray), 0,
    D3DFVF_PANELVERTEX,
    D3DPOOL_MANAGED,
    VBuffer);

  VBuffer.Lock(0, SizeOf(TPVArray), pb, 0);
  System.Move(Verts, pb^, SizeOf(TPVArray));
  VBuffer.Unlock;

  SetPos(Bounds.Left, Bounds.Top);

end;

procedure TDXPanel.Render;
var
  src, dest: dword;

begin
  if not Visible then Exit;
  inherited;
  case Blend of
    bl_normal:
      begin
        SRC := D3DBLEND_SRCALPHA;
        DEST := D3DBLEND_INVSRCALPHA;
      end;
    bl_add:
      begin
        SRC := D3DBLEND_ONE;
        DEST := D3DBLEND_ONE;
      end;

  end;
  Device.SetRenderState(D3DRS_SRCBLEND, SRC);
  Device.SetRenderState(D3DRS_DESTBLEND, DEST);
  Device.SetVertexShader(D3DFVF_PANELVERTEX);



  Device.SetStreamSource(0, VBuffer, sizeof(TPANELVERTEX));
  Device.DrawPrimitive(D3DPT_TRIANGLESTRIP, 0, 2);


 { Device.SetRenderState(D3DRS_FILLMODE,D3DFILL_WIREFRAME );

  OVerts:=Verts;
  OVerts[0].color:=$FFFFAA00;
  OVerts[1].color:=$FFFFAA00;
  OVerts[2].color:=$FFFFAA00;
  OVerts[3].color:=$FFFFAA00;
  Device.SetTexture(0,nil);
  Device.DrawPrimitiveUP(D3DPT_TRIANGLESTRIP,2,@OVerts,SizeOf(TpanelVertex));


  Device.SetRenderState(D3DRS_FILLMODE,D3DFILL_SOLID );}


end;


procedure TDXPanel.SetPos(x, y: long);
begin
  D3DXMatrixTranslation(PosMatrix, x + (Bounds.Right div 2), y + (Bounds.Bottom div 2), 0);
  Bounds.Left := GetPos.x;
  Bounds.Top := GetPos.Y;

end;


function TDXPanel.GetPos: TPoint;
begin
  Result := Point(round(Posmatrix.m[3, 0]) - (Bounds.Right div 2),
    round(Posmatrix.m[3, 1] - (Bounds.Bottom div 2)));


end;

function TDXPanel.GetRealPos:TVector2;
begin
 Result := Vec2(Posmatrix.m[3, 0] - (Bounds.Right div 2),
    Posmatrix.m[3, 1] - (Bounds.Bottom div 2));

end;

procedure TDXPanel.RotateAbs(angle: single);
begin
  RotateAbs2D(angle);
end;

procedure TDXPanel.RotateRel(angle: single);
begin
  RotateRel2D(angle);
end;

procedure TDXPanel.Move(x, y: single);
begin
  Move2D(x, y);
end;

procedure TDXPanel.ScaleAbs(x, y: single);
begin
  ScaleAbs2D(x, y);
end;

procedure TDXPanel.ScaleRel(x, y: single);
begin
  ScaleRel2D(x, y);
end;

procedure TDXPanel.Resize(w, h: long);
var Pos:TPoint;
begin
  Bounds := Rect(Bounds.Left, Bounds.Top, w, h);

  Pos := GetPos;
  Init;
  SetPos(Pos.X, Pos.Y);

end;

procedure TDXPanel.Center(x, y: boolean);
begin
  if X then SetPos((Bounds.Right - GetPos.X) div 2, GetPos.Y);
  if Y then SetPos(GetPos.X, (Bounds.Bottom - GetPos.Y) div 2);


end;

procedure TDXPanel.SetColor(col: TD3DColor);
var
  Pos: TPoint;
  i: long;
begin
  Color := col;
  for i := 0 to 3 do
    Verts[i].color := Color;

  Pos := GetPos;
  Init;
  SetPos(Pos.X, Pos.Y);

end;


////////////////////////  TDXPANELEX
procedure TDXPanelEx.DoTimer;
begin
 NextFrame;

end;             

procedure TDXPanelEx.NextFrame;
begin
if not CanAnim then Exit;

if GetTickCount>Tag2+1000/Rate then 
 begin
  Tag2:=GetTickCount;
  inc(AFrame);
  if AFrame>Frames-1 then AFrame:=0;
  SetPosUV(AFrame*step_u+0.01,AFrame*step_v+0.01,(AFrame+1)*step_u-0.01,(AFrame+1)*step_v-0.01);
  
 end;

end;

procedure TDXPanelEx.SetPosUV(minu, minv, maxu, maxv: single);
var Pos:TPoint;
begin
Pos := GetPos;
SetUV(minu, minv, maxu, maxv);
SetPos(Pos.X, Pos.Y);

end;

procedure TDXPanelEx.SetAnim(w,h,r:long;can:boolean);
var Desc:TD3DSurface_Desc;

begin
if (w=0)or(h=0)or(r=0)or(Texture.Tex=nil) then Exit;

FWidth:=w;
FHeight:=h;
Rate:=r;
CanAnim:=can;

Texture.Tex.GetLevelDesc(0,Desc);

Frames:=Desc.Width div w;
AFrame:=0;


step_u:=FWidth/Desc.Width;
step_v:=FHeight/Desc.Height;


Tag2:=GetTickCount;

if not CanAnim then Exit;

SetPosUV(MIN_U,MIN_V,step_u,step_v);



end;







initialization

  ZeroMemory(@defmtrl, sizeof(TD3DMATERIAL8));
  defmtrl.Diffuse.r := 1;
  defmtrl.Ambient.r := 1;
  defmtrl.Specular.r:=1;
  defmtrl.Emissive.r:=1;


  defmtrl.Diffuse.g := 1;
  defmtrl.Ambient.g := 1;
  defmtrl.Specular.g :=1;
  defmtrl.Emissive.r:=1;


  defmtrl.Diffuse.b := 1;
  defmtrl.Ambient.b := 1;
  defmtrl.Specular.b:=1;
  defmtrl.Emissive.r:=1;


  defmtrl.Diffuse.a := 1;
  defmtrl.Ambient.a := 1;
  defmtrl.Specular.a:=1;
  defmtrl.Emissive.r:=1;

  defmtrl.Power:=5;

 RegisterClasses([TDXObj,TDXPanel]);


end.
