{
  Drake Rider

  (C) 2002 Vampyre SoftWorks
  Created by :  Marek Mauder  
}	


unit EngineMain;

interface

uses Windows, Classes, Sysutils, DirectXGraphics,Graphics,
  D3DX8, DXObj, gameTypes, WinFunc, EngineObj,GUI,D3DFunc,
  ContNRS, DXStuff, Sound, maingame,Maps,Engequip;

const

  //// Engine speeds
  ES_10FPS = 10;
  ES_30FPS = 30;
  ES_60FPS = 60;
  ES_100FPS =100;
  ES_UNFPS = $7FFFFFFF;

type

  TEngine=class;

  TOnMapEnd=procedure(Eng:TEngine)of Object;


  TEngine = class
  public
    Device: IDevice;
    Dim: TPoint;

    Objects: TBaseObjList;

    WorldX,
      WorldY: long;
    Speed: single;

    Perf: long;

    Stopped:boolean;
    MoveStopped:boolean;

    Tag1,
      Tag2,
      Tag3: long;

    Map:TMap;

    Ground: long;
    Sky:long;
    MaxWorldX:long;

    OnMapEnd:TOnMapEnd;

    MsgBox:TDXPanelEx;
    MsgTxt:TGuiObj;
    PlInfos:array[0..3]of TDXPanelEx;
    Plnames:array[0..3]of TGuiObj;

    Players:array[0..3]of TPlayer;

    Adding:boolean;

    AMsg,LastMsg:string;

    InfoFont:ID3DXFont;

    constructor Create(ADev: iDevice; ADim: TPoint); virtual;
    destructor Destroy; override;

    function Add(Obj: TbaseObj): TBaseObj;

    procedure Render; virtual;
    procedure DrawHud;virtual;
    procedure CheckInput; virtual;
    procedure DoTimer; virtual;
    procedure Clear;

    procedure LoadMap(MapName:string);

    procedure Stop;
    procedure Resume;

    procedure StopMove;
    procedure ResumeMove;

    function CreateObjFromInfo(I:TBaseInfo):TBaseObj;

    procedure DoMapEnd;

    procedure AddMsg(msg:string);

    procedure SetPerf(pf:long);

  end;






implementation


constructor TEngine.Create(ADev: iDevice; ADim: TPoint);
var i:long;
begin
  inherited Create;
  Device := ADev;
  Dim := ADim;
  Objects := TBaseObjList.Create;
  Map:=TMap.Create;
  Perf := ES_30FPS;
  tag3 := GetTickCount;

  Speed := 1.0;

  WorldX := 0;
  WorldY := 0;
  Ground := 20;
  Sky:=30;

  WeaponMgr:=TWeaponMgr.Create;
  WeaponMgr.Load;
  ArmorMgr:=TArmorMgr.Create;
  ArmorMgr.Load;

  Adding:=true;

  Resume;

  end;

destructor TEngine.Destroy;
begin
  Map.Free;
  Clear;
  Device := nil;
  Objects.Free;
  inherited;
end;

procedure TEngine.SetPerf(pf:long);
begin
 case pf of
 1:Perf:=ES_10FPS;
 2:Perf:=ES_30FPS;
 3:Perf:=ES_60FPS;
 4:Perf:=ES_100FPS;
 5:Perf:=ES_UNFPS;
 end;

end;

procedure TEngine.Render;
var
  i, imax: long;
  old: dword;
begin
//Device.SetRenderState(D3DRS_LIGHTING, ord(TRUE));
  Device.SetRenderState(D3DRS_AMBIENT, $FFDDDDDD);




  for i := 0 to Objects.Count - 1 do
    Objects[i].Render;

  DrawHud;
end;

function TEngine.Add(Obj: TbaseObj): TBaseObj;
var
  L, H, I, C: Integer;
begin
  L := 0;
  H := Objects.Count - 1;
  while L <= H do
  begin
    I := (L + H) div 2;
    C := TBaseObj(Objects[I]).Z - Obj.Z;
    if C < 0 then
      L := I + 1
    else
      H := I - 1;
  end;
  Obj.Engine := Self;
  Objects.Insert(L, Obj);
  Result := Obj;
end;

procedure TEngine.CheckInput;
var
  i: long;
begin
  for i := 0 to Objects.Count - 1 do
    Objects[i].CheckInput;


end;

procedure TEngine.DoTimer;
var
  i, imax: long;
begin
  if  Stopped then Exit;

  if GetTickCount > Tag3 + 1000 div Perf then
  begin
    Tag3 := GetTickCount;

    imax := Objects.Count - 1;
    for i := 0 to imax do
      if i >= Objects.Count then
        break
      else if Objects[i].Deaded then
        Objects.Remove(Objects[i]);

    CheckInput;

    if not MoveStopped then
    WorldX:=round(WorldX+Abs(SP_WORLD));


    for i := 0 to Objects.Count - 1 do
      Objects[i].DoMove;


    //if WorldX>MaxWorldX-Dim.X then StopMove;
    if WorldX>MaxWorldX then DoMapEnd;

  end;

end;

procedure TEngine.Clear;
var i,imax:long;

begin
  Stop;

  Objects.Clear;


  TexMgr.RemoveTag(LT_LEVEL);
  SoundMgr.RemoveTag(LT_LEVEL);

  for i:=0 to 3 do
   begin Players[i]:=nil;end;


  InfoFont:=nil;
  aMsg:=' ';
  lastMsg:=' ';
end;

procedure TEngine.LoadMap(MapName:string);
var F:TLoadFile;
    MS:TMS;
    i:long;
begin
 Clear;

 F:=LoadFile(Exedir+'Data\Maps\'+MapName);
 MS:=TMS.Create;
 MS.WriteBuffer(F.Bytes^,F.Size);
 MS.Size:=F.Size;
 MS.Position:=0;

 Map.LoadFromStream(MS);

 MS.Free;
 FreeFile(F);


 for i:=0 to Map.Infos.Count-1 do
  Add(CreateObjFromInfo(Map.Infos[i]));

 WorldX:=0;
 WorldY:=0;

 Sky:=Map.Sky;
 Ground:=Map.Ground;
 MaxWorldX:=Map.MaxWorldX;

 Resume;
end;


procedure TEngine.Stop;
begin
 Stopped:=true;

end;

procedure TEngine.Resume;
begin
 Stopped:=false;

end;

procedure TEngine.StopMove;
begin
 MoveStopped:=true;

end;

procedure TEngine.ResumeMove;
begin
 MoveStopped:=false;

end;

function TEngine.CreateObjFromInfo(I:TBaseInfo):TBaseObj;
var O:TBaseObj;
    E:tEnemyinfo;
    cls:string;
begin
 cls:=I.ClassName;
 Delete(cls,Length(I.ClassName)-3,4);
 O:=TEngObjClass(FindClass(cls)).Create(Device,
 Rect(I.X,I.Y,I.Width,I.Height),TexMgr.GetTex(I.TexName),
  I.Color, I.Z, true);

 O.LoadFromInfo(I);


 Result:=O;
end;

procedure TEngine.DoMapEnd;
begin
 if Assigned(OnMapEnd) then OnMapEnd(Self);
end;

procedure TEngine.DrawHud();
var i:long;
    modf,modw,hpmod:single;
    F:TFont;
    m:td3ddisplaymode;
    R1,R2:TRect;
begin

if MsgBox=nil then
 begin
  MsgBox:=TDXpanelEX.Create(Device,Rect(267,Dim.Y-128,267,128),$77FFFFFF,TexMgr.GetTex('ingame\msgbox.png'));
  MsgTxt:=TGuiObj.Create(Device,'msgtxt',Rect(267+10,Dim.Y-128+5,267-20,128-10),$FFFFFFFF,TexMgr.GetTex('ingame\msgtxt.png'));


  for i:=0 to 3 do
  begin
  PlInfos[i]:=TDXPanelEx.Create(Device,Rect(0,0,267,64),$47FFFFFF,TexMgr.Gettex('ingame\plinfo.png'));
  PlNames[i]:=TGuiObj.Create(Device,'Plnm'+inttostr(i+1),Rect(0,0,256,16),$FFDDDDDD,TexMgr.Gettex('ingame\plname.png'));
  end;
  PlInfos[0].SetPos(0,Dim.Y-128);
  PlInfos[1].SetPos(534,Dim.Y-128);
  PlInfos[2].SetPos(0,Dim.Y-66);
  PlInfos[3].SetPos(534,Dim.Y-66);

  Plnames[0].SetPos(5,Dim.Y-128+2);
  Plnames[1].SetPos(534+5,Dim.Y-128+2);
  Plnames[2].SetPos(5+5,Dim.Y-66+2);
  Plnames[3].SetPos(534+5,Dim.Y-66+2);


 end;


modf:=maingame.d3dpp.BackBufferWidth / Dim.X;
modw:=modf;
if modf=0 then
begin
modf:=gamewin.clientWidth / Dim.X;
modw:=GameWin.ClientHeight/Dim.Y;
end;

if InfoFont=nil then
begin
 F:=TFont.Create;
 F.Name:='Calligraphia';
  F.Color:=RGBATORGB(FNT_INGCOL);
 F.Style:=[fsBold];


 F.Size:=round(9*modf);
 D3DXCreateFont(Device,F.Handle,InfoFont);
 modw:=f.Size;

 F.Free;
end;

MsgBox.Render;
MsgTxt.Render;



R1:=Rect(round(modf*(267+10)),round(modw*(Dim.Y-128+8)),round(modf*(534-20)),round(modw*(Dim.Y-5)));
InfoFont.DrawTextA(Pchar(Pchar(lastmsg)+#13+#13+Pchar(AMsg)),-1,R1,DT_WORDBREAK,FNT_INGCOL);



for i:=0 to 3 do
 begin
 if Players[i]=nil then Continue;
 PlInfos[i].Render;

 hpmod:=Players[i].HP / (Players[i].MaxHP);

 R2:=Rect(round((PlNames[i].GetPos.X)*modf),
        round((PLNames[i].GetPos.Y)*modw),
        round((PLNames[i].GetPos.X+PlNames[i].Bounds.Right*hpmod)*modf),
        round((PLNames[i].GetPos.Y+PlNames[i].Bounds.Bottom)*modw));


 _D3DR:=D3DRFromRect(R2);
 Device.Clear(1,@_D3DR,D3DCLEAR_TARGET,Players[i].ImgCol,1,0);

 PlNames[i].Render;


 R1:=Rect( round((PlNames[i].GetPos.X+5)*modf),round((PLNames[i].GetPos.Y+16)*modw),round((PlNames[i].GetPos.X+5+128)*modf),round((PLNames[i].GetPos.Y+16+14)*modw));
 InfoFont.DrawTextA(Pchar(
        Format('ivoty: %d/%d ',[Players[i].HP,Players[i].MaxHP])
        ),-1,R1,DT_LEFT,$FFFF3300);

 R1:=Rect( round((PlNames[i].GetPos.X+5+128)*modf),round((PLNames[i].GetPos.Y+16)*modw),round((PlNames[i].GetPos.X+5+256)*modf),round((PLNames[i].GetPos.Y+16+14)*modw));
 InfoFont.DrawTextA(Pchar(
        Format('Zku.: %d XP ',[Players[i].XP])
        ),-1,R1,DT_RIGHT,$FF00bbFF);


 R1:=Rect( round((PlNames[i].GetPos.X+5)*modf),round((PLNames[i].GetPos.Y+16+16)*modw),round((PlNames[i].GetPos.X+5+256)*modf),round((PLNames[i].GetPos.Y+16+16+14+16)*modw));
 InfoFont.DrawTextA(Pchar(
        Format('Zbran: %s, %s, %s',[Players[i].Ranged.Name,Players[i].Melee.Name,Players[i].RangedB.Name])
        ),-1,R1,DT_WORDBREAK,fnt_ingcol);


 end;
 

end;

procedure TEngine.AddMsg(msg:string);
var List:TStringList;
 begin
 {
  // ni pouziti,trva moc dlouho
  List:=tStringList.Create;
  if LastMsg='' then LastMsg:=' ';
  List.Add(lastMsg);
  List.Add('#'+Msg);
  if adding then  MsgTxt.AddStrings(List,Fonts[FN_ING],false,false,true);
  List.Free;


  }

  lastMsg:=AMsg;

  Amsg:=msg;



 end;

end.

