{
  Drake Rider

  (C) 2002 Vampyre SoftWorks
  Created by :  Marek Mauder  
}	


unit GUI;

interface

uses Windows, Classes, SysUtils, DirectXGraphics, KeyBoard,
  D3DX8, gameTypes, D3DFunc, DXObj, WinFunc,
  Messages, DXStuff, MainGame, Graphics,Sound,Types;

type
  TGuiObj = class;
  TGuiWin = class;
  TGuiObjList = class;
  TGuiWinList = class;

  TGMSG = record
    Win: TGuiWin;
    Obj: TGuiObj;
    Data: string;
    MSG: dword;
    Pos: TPoint;
  end;




  TGuiObj = class(TDXPanelEx)
  private


  public
    Owner: TGuiWin;
    Enabled: boolean;
    CanAct: boolean;

    OrigTx: string;

    OrigColor: TD3DColor;
    MouseColor: TD3DColor;
    ClickColor: TD3DColor;

    MouseDown: boolean;
    Fading: boolean;
    FIn: boolean;

    oldE: boolean;
    oldB: TDXBlend;
    Hafter: boolean;

    constructor Create(ADev: IDevice; AName: string;
      ABounds: TRect; AColor: TD3DColor; Tex: TDXTexture = nil;
      TexName: string = ''; AInit: boolean = true;
      ITex: ITexture = nil); virtual;
    destructor Destroy; override;

    procedure MouseStart; override;
    procedure MouseStop; override;
    function MouseIn: boolean;
    procedure DoTimer; override;
    function GetRect: TREct;
    procedure Send(id: dword); virtual;
    procedure SetColors(orig, mouse, click: TD3DColor); virtual;
    procedure StartFade(AFIn: boolean; Hideafterfout: boolean = true); virtual;
    procedure StopFade; virtual;
    procedure AddCaption(Txt: string; FI: TFontInfo; Pos: TPoint; Center: boolean
      = true); virtual;
    procedure AddStrings(Str: TStrings; FI: TFontInfo; CenterLines: boolean =
      false; Center: boolean = false; Wrap: boolean = false; LineBreak: long =
      $FF); virtual;
    procedure Show; override;

  end;

  TGuiWin = class(TGuiObj)
  private



  public
    Objects: TGUIObjList;


    constructor Create(ADev: IDevice; AName: string;
      ABounds: TRect; AColor: TD3DColor; Tex: TDXTexture = nil;
      TexName: string = ''; AInit: boolean = true;
      ITex: ITexture = nil); override;
    destructor Destroy; override;

    function Add(Obj: TGuiObj): TGuiObj;
    function Find(AName: string): TGuiObj;
    procedure ToFront; virtual;
    procedure Render; override;

    procedure Move(x, y: single); override;
    procedure SetPos(x, y: long); override;
    procedure DoTimer; override;
    procedure Send(id: dword); override;
    procedure StartFade(AFIn: boolean; all: boolean = true; Hideafterfout:
      boolean = true); reintroduce;
    procedure StopFade(all: boolean = true); reintroduce;

    procedure Show; override;
  end;

  TGuiDesktop = class(TGuiWin)
  end;

  TPerformProc = procedure(MSG: TGMSG) of object;

  TGuiController = class
  public
    Device: IDevice;

    Desktop: TGuiDesktop;
    Windows: TGuiWinList;

    Dim: TPoint;

    Active: TGuiObj;
    Mouse: TGuiObj;

    PerformProc: TPerformProc;
    CanInput: boolean;

    MWin: TGuiWin;
    MBtn: TGuiObj;
    Modal: boolean;
    MData: string;

    constructor Create(ADev: IDevice; ADim: TPoint; PP: TPerformProc = nil);
    destructor Destroy; override;

    function Add(Win: TGuiWin): TGuiWin;
    function Find(AName: string): TGuiWin;
    procedure Render;
    procedure CheckInput(Msg: TMsg);
    function GetMouse: TGuiObj;
    procedure DoTimer;
    procedure Perform(MSG: TGMSG);
    function ShowModal(Win: TGuiWin; Btn: TGuiObj; Data: string = ''): boolean;


    procedure HideAll;
    procedure Clear;

  end;


  TGuiObjList = class(TList)
  private
    FOwnsObjects: Boolean;
  protected
    procedure Notify(Ptr: Pointer; Action: TListNotification); override;
    function GetItem(Index: Integer): TGuiObj;
    procedure SetItem(Index: Integer; AGuiObj: TGuiObj);
  public
    constructor Create; overload;
    constructor Create(AOwnsObjects: Boolean); overload;

    function Add(AGuiObj: TGuiObj): Integer;
    function Remove(AGuiObj: TGuiObj): Integer;
    function IndexOf(AGuiObj: TGuiObj): Integer;
    procedure Insert(Index: Integer; AGuiObj: TGuiObj);
    property OwnsObjects: Boolean read FOwnsObjects write FOwnsObjects;
    property Items[Index: Integer]: TGuiObj read GetItem write SetItem; default;
  end;

  TGuiWinList = class(TList)
  private
    FOwnsObjects: Boolean;
  protected
    procedure Notify(Ptr: Pointer; Action: TListNotification); override;
    function GetItem(Index: Integer): TGuiWin;
    procedure SetItem(Index: Integer; AGuiWin: TGuiWin);
  public
    constructor Create; overload;
    constructor Create(AOwnsObjects: Boolean); overload;

    function Add(AGuiWin: TGuiWin): Integer;
    function Remove(AGuiWin: TGuiWin): Integer;
    function IndexOf(AGuiWin: TGuiWin): Integer;
    procedure Insert(Index: Integer; AGuiWin: TGuiWin);
    property OwnsObjects: Boolean read FOwnsObjects write FOwnsObjects;
    property Items[Index: Integer]: TGuiWin read GetItem write SetItem; default;
  end;


function MSGCr(id: dword; Obj: TGuiObj): TGMSG;


implementation



function MSGCr(id: dword; Obj: TGuiObj): TGMSG;
var
  M: TGMSG;
begin
  M.Obj := Obj;
  if not (Obj is TGuiWin) then
    M.Win := TGuiWin(Obj.Owner)
  else
    M.Win := TGuiWin(Obj);
  M.Pos := Cursor.GetPos;
  M.MSG := id;

  Result := M;
end;





//////////////////  TGUIOBJLIST



function TGuiObjList.Add(AGuiObj: TGuiObj): Integer;
begin
  Result := inherited Add(AGuiObj);
end;



constructor TGuiObjList.Create;
begin
  inherited Create;
  FOwnsObjects := True;
end;

constructor TGuiObjList.Create(AOwnsObjects: Boolean);
begin
  inherited Create;
  FOwnsObjects := AOwnsObjects;
end;


function TGuiObjList.GetItem(Index: Integer): TGuiObj;
begin
  Result := inherited Items[Index];
end;

function TGuiObjList.IndexOf(AGuiObj: TGuiObj): Integer;
begin
  Result := inherited IndexOf(AGuiObj);
end;

procedure TGuiObjList.Insert(Index: Integer; AGuiObj: TGuiObj);
begin
  inherited Insert(Index, AGuiObj);
end;

procedure TGuiObjList.Notify(Ptr: Pointer; Action: TListNotification);
begin
  if OwnsObjects then
    if Action = lnDeleted then
      TGuiObj(Ptr).Free;
  inherited Notify(Ptr, Action);
end;

function TGuiObjList.Remove(AGuiObj: TGuiObj): Integer;
begin
  Result := inherited Remove(AGuiObj);
end;

procedure TGuiObjList.SetItem(Index: Integer; AGuiObj: TGuiObj);
begin
  inherited Items[Index] := AGuiObj;
end;

//////////////////  TGUIWINLIST


function TGuiWinList.Add(AGuiWin: TGuiWin): Integer;
begin
  Result := inherited Add(AGuiWin);
end;



constructor TGuiWinList.Create;
begin
  inherited Create;
  FOwnsObjects := True;
end;

constructor TGuiWinList.Create(AOwnsObjects: Boolean);
begin
  inherited Create;
  FOwnsObjects := AOwnsObjects;
end;


function TGuiWinList.GetItem(Index: Integer): TGuiWin;
begin
  Result := inherited Items[Index];
end;

function TGuiWinList.IndexOf(AGuiWin: TGuiWin): Integer;
begin
  Result := inherited IndexOf(AGuiWin);
end;

procedure TGuiWinList.Insert(Index: Integer; AGuiWin: TGuiWin);
begin
  inherited Insert(Index, AGuiWin);
end;

procedure TGuiWinList.Notify(Ptr: Pointer; Action: TListNotification);
begin
  if OwnsObjects then
    if Action = lnDeleted then
      TGuiWin(Ptr).Free;
  inherited Notify(Ptr, Action);
end;

function TGuiWinList.Remove(AGuiWin: TGuiWin): Integer;
begin
  Result := inherited Remove(AGuiWin);
end;

procedure TGuiWinList.SetItem(Index: Integer; AGuiWin: TGuiWin);
begin
  inherited Items[Index] := AGuiWin;
end;


//////////////////////////////  TGUIOBJ


constructor TGuiObj.Create(ADev: IDevice; AName: string;
  ABounds: TRect; AColor: TD3DColor; Tex: TDXTexture = nil;
  TexName: string = ''; AInit: boolean = true;
  ITex: ITexture = nil);
begin
  inherited Create(ADev, ABounds, AColor, Tex, TexName, AInit, ITex);
  Name := AName;
  Enabled := true;
  if AColor = 0 then
    OrigColor := BTN_OrigCol
  else
    OrigColor := AColor;
  MouseColor := BTN_MouseCol;
  ClickColor := BTN_ClickCol;
  SetColor(OrigColor);
  CanAct := true;
  OrigTx := Texture.Name;
end;

destructor TGuiObj.Destroy;
begin


  inherited;
end;

procedure TGuiObj.MouseStart;
begin
  if (not Enabled) or (not CanAct) then Exit;
  SetColor(MouseColor);
  Send(MSG_MOUSEENTER);


end;

procedure TGuiObj.MouseStop;
begin
  if (not Enabled) or (not CanAct) then Exit;
  Send(MSG_MOUSELEAVE);

  SetColor(OrigColor);
  if MouseDown then
  begin
    MouseDown := false;
//    if not (Self is TGuiWin) then Move(-BTN_TRANSLATE, -BTN_TRANSLATE);
  end;

end;

procedure TGuiObj.DoTimer;
var
  al, oal: dword;
begin
  inherited;

  if Fading then
  begin
    al := RGBA_GETA(Color);
    oal := RGBA_GETA(OrigColor);
    if GetTickCount > Tag3 + FADE_INT then
    begin
      if not FIn then
      begin

        if al <= FADE_POWER then
          al := 0
        else
          al := al - FADE_POWER;

        SetColor(D3DCOLOR_ARGB(al, RGBA_GETR(Color), RGBA_GETG(Color),
          RGBA_GETB(Color)));
        if al = 0 then StopFade;

      end
      else
      begin

        if al >= oal - FADE_POWER then
          al := oal
        else
          al := al + FADE_POWER;

        SetColor(D3DCOLOR_ARGB(al, RGBA_GETR(Color), RGBA_GETG(Color),
          RGBA_GETB(Color)));
        if al = oal then StopFade;



      end;
      Tag3 := GetTickCount;
    end;
  end;


  if MouseIn and Enabled then
  begin

   ///////// MOUSE RELEASED [CLICK]
    if MouseDown and (not IsKeyDown(VK_LBUTTON)) then
    begin
      if not(Self is TGuiWin) then SoundMgr.GetSnd('gui\butclick.wav').Play;

      Send(MSG_CLICKED);
      Send(MSG_MOUSEUP);
      if CanAct then SetColor(MouseColor);
  //    if CanAct then Move(-BTN_TRANSLATE, -BTN_TRANSLATE);
    end;


   //////// MOUSE CLICKED DOWN
    if (not MouseDown) and IsKeyDown(VK_LBUTTON) then
    begin
      Send(MSG_MOUSEDOWN);
      if CanAct then SetColor(ClickColor);
//      if CanAct then Move(BTN_TRANSLATE, BTN_TRANSLATE);


    end;

   /////// CONTROL MOUSE STATE
    MouseDown := IsKeyDown(VK_LBUTTON);
    if MouseDown then
    begin
     //ClickColor;

    end;


  end;



end;

procedure TGuiObj.Send(id: dword);
begin
  case id of
    MSG_CLICKED:


  end;

  if Self is TGuiWin then
    TGuiController(Owner).Perform(MSGCr(id, Self))
  else
    TGuiController(Owner.Owner).Perform(MSGCr(id, Self));

end;


procedure TGuiObj.SetColors(orig, mouse, click: TD3DColor);
begin
  OrigColor := orig;
  MouseColor := mouse;
  ClickColor := click;
  SetColor(orig);
end;

procedure TGuiObj.StartFade(AFIn: boolean; Hideafterfout: boolean);
begin
  Fading := true;
  FIn := AFin;
  MouseDown := false;

  oldE := Enabled;
  oldB := Blend;
  Blend := bl_normal;
  Enabled := false;
  Hafter := Hideafterfout;
  Show;

  if FIn then
    SetColor(D3DCOLOR_ARGB(0, RGBA_GETR(OrigColor), RGBA_GETG(OrigColor),
      RGBA_GETB(OrigColor)))
  else
    SetColor(OrigColor);
  Tag3 := GetTickCount;
end;

procedure TGuiObj.StopFade;
begin
  if not FIn then
    Send(MSG_FADEDOUT)
  else
    Send(MSG_FADEDIN);

  if not Fin and Hafter then Hide;

  SetColor(OrigColor);
  fading := false;
  Enabled := oldE;
  Blend := oldB;


end;

procedure TGuiObj.AddCaption(Txt: string; FI: TFontInfo; Pos: TPoint; Center:
  boolean);

var
  TX: ITexture;
  DXTX: TDXTexture;
  Btx: TBitmap;
  texname: string;
  fH, fW: long;
  C: TCanvas;
  s: string;
  ckey: tD3DColor;
  R1, R2, R: TRect;
begin
  texname := Name + '-capt tex';


  Btx := BMPFromTex(TexMgr.GetTex(origtx).Tex);
  C := bTX.Canvas;
  C.Font.Color := RGB(rgba_getr(FI.Color), rgba_getg(FI.Color),
    rgba_getb(FI.Color));
  C.Font.Name := FI.Name;
  C.Font.Size := FI.Size;
  C.Font.Style := FI.Style;
  SetBKMode(C.Handle, Transparent);

  fH := C.TextHeight(Txt);
  fW := C.TextWidth(Txt);
  R1 := Rect(2, (BTx.Height - fh) div 2, btx.width - 2, (BTx.Height - fh) div 2
    + fh);
  R2 := Rect(Pos.X, Pos.Y, Pos.X + fw, Pos.Y + fh);
  if Center then
    R := R1
  else
    R := R2;


  DrawText(C.Handle, Pchar(Txt), -1, R, DT_CENTER);

  if TexMgr.GetTex(origtx).Colorkey <> 0 then
    ckey := $FF000000
  else
    ckey := 0;

  TexMgr.RemoveTex(texname);
  TX := TexFromBMP(Btx, Ckey);
  DXTX := TDXTexture.Create(Device, texname);
  DXTX.Tex := TX;
  TexMgr.AddTex(DXTX);

  Texture := nil;
  Texture := TexMgr.Find(texname);
  Btx.Free;

end;

procedure TGuiObj.AddStrings(Str: TStrings; FI: TFontInfo; CenterLines: boolean;
  Center: boolean; Wrap: boolean; LineBreak: long);
var
  iTX: ITexture;
  DXTX: TDXTexture;
  Btx, Bsm: TBitmap;
  texname: string;
  flag, fT, fH, fW, up, fwmax: long;
  sx, sy, tx, ty: long;
  xf, yf: single;
  C: TCanvas;
  s: string;
  i: long;
  Buff: Pchar;
  ckey: tD3DColor;
  R1, R2, R: TRect;
begin
  texname := Name + '-capt tex';
  if Str = nil then Exit;
  if Str.Count = 0 then Exit;

  Btx := BMPFromTex(TexMgr.GetTex(origtx).Tex);

  C := bTX.Canvas;
  C.Font.Color := RGB(rgba_getr(FI.Color), rgba_getg(FI.Color),
    rgba_getb(FI.Color));
  C.Font.Name := FI.Name;
  C.Font.Size := FI.Size;
  C.Font.Style := FI.Style;
  fH := C.TextHeight('T');
  if LineBreak = $FF then LineBreak := fH div 10;
  fT := Str.Count * fh + (Str.Count) * Linebreak;
  fwmax := 0;
  for i := 0 to Str.Count - 1 do
    if C.TextWidth(Str[i]) > fwmax then fwmax := C.TextWidth(Str[i]);
  SetBKMode(C.Handle, Transparent);


  if not Wrap then
  begin
    up := 1;
    if center then up := (Btx.Height - fT) div 2;

    for i := 0 to Str.Count - 1 do
    begin

      fW := C.TextWidth(Str[i]);

      R := Rect(2, up + fH * i + i * LineBreak, btx.Width - 2, up + fH * i + i *
        LineBreak + fh);

      flag := DT_VCENTER;
      if CenterLines then flag := flag or DT_CENTER;

      DrawText(C.Handle, Pchar(Str[i]), -1, R, flag);

    end;
  end
  else
  begin
    Buff := '';
    for i := 0 to Str.Count - 1 do
    begin
      s := Str[i];
      if Length(s) > 0 then
        if s[1] = '#' then
        begin
          delete(s, 1, 1);
          Buff := Pchar(buff + #13 + s + ' ');
        end
        else
          Buff := Pchar(Buff + Str[i] + ' ');
    end;
    R := Rect(2, 2 + Linebreak, btx.Width - 2, btx.height - 2);

    flag := DT_VCENTER or DT_WORDBREAK;
    DrawText(C.Handle, Buff, -1, R, flag);
  end;

  if TexMgr.GetTex(origtx).Colorkey <> 0 then
    ckey := $FF000000
  else
    ckey := 0;

  TexMgr.RemoveTex(texname);
  iTX := TexFromBMP(Btx, Ckey);
  DXTX := TDXTexture.Create(Device, texname);
  DXTX.Tex := iTX;
  TexMgr.AddTex(DXTX);

  Texture := nil;
  Texture := TexMgr.Find(texname);
  Btx.Free;
end;


procedure TGuiObj.Show;
begin
  inherited;
  Send(MSG_SHOW);
end;

/////////////////////  TGUIWIN


constructor TGuiWin.Create(ADev: IDevice; AName: string;
  ABounds: TRect; AColor: TD3DColor; Tex: TDXTexture = nil;
  TexName: string = ''; AInit: boolean = true;
  ITex: ITexture = nil);
begin
  inherited;
  Objects := TGuiObjList.Create;
  if AColor = 0 then
    OrigColor := WIN_OrigCol
  else
    OrigColor := AColor;
  MouseColor := WIN_MouseCol;
  ClickColor := WIN_ClickCol;
  SetColor(OrigColor);
  CanAct := false;
  Hide;

end;

destructor TGuiWin.Destroy;
begin

  Objects.Free;
  inherited;
end;

function TGuiWin.Add(Obj: TGuiObj): TGuiObj;
begin
  Objects.Add(Obj);
  Obj.Owner := Self;
  Obj.SetPos(GetPos.X + Obj.GetPos.X, GetPos.Y + Obj.GetPos.Y);

  Result := Obj;

end;

procedure TGuiWin.Show;
begin
  inherited;
  ToFront;

end;


function TGuiWin.Find(AName: string): TGuiObj;
var
  i: long;
begin
  Result := nil;
  for i := 0 to Objects.Count - 1 do
    if CompStr(AName, TGuiObj(Objects[i]).Name) then
      Result := TGuiObj(Objects[i]);

end;


procedure TGuiWin.ToFront;
var
  Wins: TGuiWinList;
begin
  if Self is TGuiDesktop then Exit;
  if not Enabled then Exit;
  Wins := TGuiController(owner).Windows;
  Wins.OwnsObjects := false;
  Wins.Remove(Self);
  Wins.Add(Self);
  Wins.OwnsObjects := true;

end;



procedure TGuiWin.Render;
var
  i: long;
begin
  if not Visible then Exit;
  inherited;
  for i := 0 to Objects.Count - 1 do
    Objects[i].Render;

end;

function TGuiObj.GetRect: TRect;
begin
  Result := Rect(GetPos.X, GetPos.Y, GetPos.X + Bounds.Right, GetPos.Y +
    Bounds.Bottom);
end;

function TGuiObj.MouseIn: boolean;
begin
  if not (Self is TGuiWin) then
    Result := TGuiController(Owner.Owner).Mouse = Self
  else
    Result := TGuiController(Owner).Mouse = Self;


end;

procedure TGuiWin.Move(x, y: single);
var
  i: long;
begin
  inherited;
  for i := 0 to Objects.Count - 1 do
    Objects[i].Move(x, y);

end;

procedure TGuiWin.SetPos(x, y: long);
var
  i: long;
  dx, dy: array of long;
begin
  if Objects <> nil then
  begin
    SetLength(dx, Objects.Count);
    SetLength(dy, Objects.Count);

    for i := 0 to Objects.Count - 1 do
    begin

      dx[i] := Objects[i].GetPos.X - GetPos.X;
      dy[i] := Objects[i].GetPos.Y - GetPos.Y;

    end;

  end;
  inherited;
  if Objects <> nil then
    for i := 0 to Objects.Count - 1 do
    begin
      Objects[i].SetPos(GetPos.X + dx[i], GetPos.Y + dy[i]);

    end;

end;


procedure TGuiWin.DoTimer;
var
  i: long;
begin
  inherited;
  for i := 0 to Objects.Count - 1 do
    Objects[i].DoTimer;

end;

procedure TGuiWin.StartFade(AFIn: boolean; all: boolean = true; Hideafterfout:
  boolean = true);
var
  i: long;
begin
  inherited StartFade(AFin, Hideafterfout);
  if all then
    for i := 0 to Objects.Count - 1 do
      Objects[i].StartFade(AFin, Hideafterfout);

end;

procedure TGuiWin.StopFade(all: boolean = true);
var
  i: long;
begin
  inherited Stopfade;
  if all then
    for i := 0 to Objects.Count - 1 do
      Objects[i].StopFade;


end;


procedure TGuiWin.Send(id: dword);
begin
  inherited;
  case id of
    MSG_CLICKED: //ToFront;


  end;


end;




/////////////////////  TGUICONTROLLER


constructor TGuiController.Create(ADev: IDevice; ADim: TPoint; PP: TPerformProc
  = nil);
begin
  inherited Create;
  PerformProc := PP;
  Device := ADEv;
  Dim := ADim;
  Windows := TGuiWinList.Create;
  Desktop := TGuiDesktop.Create(Device, 'Desktop',
    Rect(0, 0, Dim.X, Dim.Y), $FF000000);
  CanInput := true;

end;



destructor TGuiController.Destroy;
begin
  Windows.Free;
  Mouse := nil;

  inherited;
end;

function TGuiController.Add(Win: TGuiWin): TGuiWin;
begin
  Windows.Add(Win);
  Win.Owner := TGuiWin(Self);
  Result := Win;
end;

function TGuiController.Find(AName: string): TGuiWin;
var
  i: long;
begin
  Result := nil;
  for i := 0 to Windows.Count - 1 do
    if CompStr(AName, Windows[i].Name) then
      Result := Windows[i];

end;

procedure TGuiController.Render;
var
  i,ii: long;
  old: dword;
begin
  Device.SetRenderState(D3DRS_LIGHTING, ord(FALSE));
  Device.SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);



  Desktop.Render;
  for i := 0 to Windows.Count - 1 do
    Windows[i].Render;



end;

procedure TGuiController.CheckInput(Msg: TMsg);
begin
  if not CanInput then Exit;

  case MSG.message of
    WM_KeyDown:
      begin
        if Modal then
          if Msg.wParam = VK_RETURN then MBtn.Send(MSG_CLICKED);



      end;
    WM_MOUSEMOVE:
      begin
        Mouse := GetMouse;

      end;

  end;
end;


function TGuiController.GetMouse: TGuiObj;
var
  ii, i: long;
  P: TPoint;

  procedure MouseEnter;
  begin
    if Result <> Mouse then
    begin
      if Mouse <> nil then Mouse.MouseStop;
      if Result <> nil then Result.MouseStart;
    end;

  end;

begin
  Result := nil;
  P := Cursor.GetPos;
  for i := Windows.Count - 1 downto 0 do
    if PtInRect(Windows[i].GetRect, P) and
      Windows[i].Visible then
      if Modal and (Windows[i] <> MWin) then
      else
      begin
        for ii := Windows[i].Objects.Count - 1 downto 0 do
          if PtInRect(Windows[i].Objects[ii].GetRect, P)
            and Windows[i].Objects[ii].Visible
            then

          begin

            Result := Windows[i].Objects[ii];
            MouseEnter;
            Mouse := Result;
            Exit;

          end;

        Result := Windows[i];
        MouseEnter;
        Mouse := Result;
        Exit;
      end;
  MouseEnter;

end;

procedure TGuiController.DoTimer;
var
  i: long;
begin

  for i := 0 to Windows.Count - 1 do
    Windows[i].DoTimer;

end;

procedure TGuiController.Perform(MSG: TGMSG);
var aMSG: TGMSG;
begin
  if Modal then
    if (Msg.Obj = MBtn) and (MWin = Msg.Win) then
      if MSG.MSG = MSG_CLICKED then
      begin

        aMsg := MsgCr(MSG_MODAL, MBtn);
        aMSG.Data := MData;
        MWin.Hide;
        Modal := false;
        MWin := nil;
        MBtn := nil;

        Perform(aMsg);


      end;
  if Assigned(PerformProc) then PerformProc(MSG);

end;

procedure TGuiController.HideAll;
var
  i: long;
begin
  for i := 0 to Windows.Count - 1 do
    Windows[i].Hide;

end;

procedure TGuiController.Clear;
begin
  Windows.Clear;
  TexMgr.RemoveTag(LT_GUI_FIXED);
  TexMgr.RemoveTag(LT_GUI_MOVABLE);


end;

function TGuiController.ShowModal(Win: TGuiWin; Btn: TGuiObj; Data: string):
  boolean;
begin
  Modal := true;
  mwin := Win;
  mbtn := Btn;
  MData := Data;

  MWin.Show;

  Result := Modal;
end;


end.

