{*********************************************************************
*
* Copyright (c) 1997-2000 Microsoft Corporation. All rights reserved
*
* File: D3DFont.h
*
* Desc: Texture-based font class
*
*********************************************************************
* timeline :
*   12/30/2k -> final version! -> all ok! haha
*   12/30/2k -> beta test with 'DrawTextScaled' SUCCESSFUL
*   12/30/2k -> beta test with 'DrawText' SUCCESSFUL !!! strike!
*   12/29/2k -> alpha build, to rebuild the nice sdk samples :)
*
* by orthkon * orthkon@mail.com * http://mp3.com/orthkon
*********************************************************************}
unit D3DFont;

interface

uses Windows, DirectXGraphics, D3DX8;

const
  // Font creation flags
  D3DFONT_BOLD      = $0001;
  D3DFONT_ITALIC    = $0002;

  // Font rendering flags
  D3DFONT_CENTERED  = $0001;
  D3DFONT_TWOSIDED  = $0002;
  D3DFONT_FILTERED  = $0004;


type
  TD3DFont = class
    m_strFontName : PChar;            // Font properties
    m_dwFontHeight : DWORD;
    m_dwFontFlags : DWORD;

    m_d3dDevice : IDirect3DDevice8;   // A D3DDevice used for rendering
    m_Texture : IDIRECT3DTEXTURE8;    // The d3d texture for this font
    m_VB : IDirect3DVertexBuffer8;    // VertexBuffer for rendering text
    m_dwTexWidth : DWORD;             // Texture dimensions
    m_dwTexHeight : DWORD;
    m_fTextScale : Single;
    m_fTexCoords : array[0..128-32] of array [0..4] of Single;

    // Stateblocks for setting and restoring render states
    m_dwSavedStateBlock : DWORD;
    m_dwDrawTextStateBlock : DWORD;
  public
    // 2D and 3D text drawing functions
    function DrawTextScaled( x, y, z, fXScale, fYScale : Real; dwColor : DWORD; strText : PChar; dwFlags : DWORD ) : HRESULT;
    function DrawText( sx, sy : Real; dwColor : DWORD; strText : PChar; dwFlags : DWORD ) : HRESULT;
    function Render3DText( strText : PChar; dwFlags : DWORD ) : HRESULT;

    // Initializing and destroying device-dependent objects
    function InitDeviceObjects( d3dDevice : IDIRECT3DDEVICE8 ) : HRESULT;
    function RestoreDeviceObjects : HRESULT;
    function InvalidateDeviceObjects : HRESULT;
    function DeleteDeviceObjects : HRESULT;

    // Function to get extent of text
    function GetTextExtent( strText : PChar; var nSize : SIZE ) : HRESULT;

    // Constructor / destructor
    constructor Create_( strFontName : PChar; dwHeight, dwFlags : DWORD );
    destructor Destroy; override;
  end;

implementation

//-----------------------------------------------------------------------------
// Custom vertex types for rendering text
//-----------------------------------------------------------------------------
type
  FONT2DVERTEX = record
    p : TD3DXVECTOR4;
    color : DWORD;
    tu, tv : Single;
  end;
  FONT3DVERTEX = record
    p, n : TD3DXVECTOR3;
    tu, tv : Single;
  end;

const
  MAX_NUM_VERTICES = 50 * 6;
  D3DFVF_FONT2DVERTEX = D3DFVF_XYZRHW or D3DFVF_DIFFUSE or D3DFVF_TEX1;
  D3DFVF_FONT3DVERTEX = D3DFVF_XYZ or D3DFVF_NORMAL or D3DFVF_TEX1;

function InitFont2DVertex( const p : TD3DXVECTOR4; color : TD3DCOLOR; tu, tv : Single ) : FONT2DVERTEX;
begin
  Result.p := p;
  Result.color := color;
  Result.tu := tu;
  Result.tv := tv;
end;

function InitFont3DVertex( const p, n : TD3DXVECTOR3; tu, tv : Single ) : FONT3DVERTEX;
begin
  Result.p := p;
  Result.n := n;
  Result.tu := tu;
  Result.tv := tv;
end;

//-----------------------------------------------------------------------------
// Name: Create()
// Desc: Font class constructor
//-----------------------------------------------------------------------------
constructor TD3DFont.Create_( strFontName : PChar; dwHeight, dwFlags : DWORD );
begin
  m_strFontName          := strFontName;
  m_dwFontHeight         := dwHeight;
  m_dwFontFlags          := dwFlags;
  m_d3dDevice            := nil;
  m_Texture              := nil;
  m_VB                   := nil;
  m_dwSavedStateBlock    := 0;
  m_dwDrawTextStateBlock := 0;
end;

//-----------------------------------------------------------------------------
// Name: Destroy()
// Desc: Font class destructor
//-----------------------------------------------------------------------------
destructor TD3DFont.Destroy;
begin
  InvalidateDeviceObjects;
  DeleteDeviceObjects;
end;

//-----------------------------------------------------------------------------
// Name: InitDeviceObjects()
// Desc: Initializes device-dependent objects, including the vertex buffer used
//       for rendering text and the texture map which stores the font image.
//-----------------------------------------------------------------------------
function TD3DFont.InitDeviceObjects( d3dDevice : IDIRECT3DDEVICE8 ) : HRESULT;
label
  PEnd;
var
  d3dCaps : TD3DCAPS8;
  Bits : Pointer;
  bmi : BITMAPINFO;
  hbm : HBITMAP;
  hdc : Windows.HDC;
  i, j, x, y, dwBold, dwItalic : DWORD;
  hFont : Windows.HFONT;
  nHeight : Integer;
  nSize : SIZE;
  str : String;
  bAlpha, c : Byte;
  d3dlr : TD3DLOCKED_RECT;
  pbits : ^WORD;
  pixels : ^DWORD;
begin
  // Keep a local copy of the device
  m_d3dDevice := d3dDevice;

  // Establish the font and texture size
  m_fTextScale  := 1.0; // Draw fonts into texture without scaling

  // Large fonts need larger textures
  if m_dwFontHeight > 40 then begin
    m_dwTexWidth := 1024;
    m_dwTexHeight := 1024;
  end else if m_dwFontHeight > 20 then begin
    m_dwTexWidth := 512;
    m_dwTexHeight := 512;
  end else begin
    m_dwTexWidth := 256;
    m_dwTexHeight := 256;
  end;

  // If requested texture is too big, use a smaller texture and smaller font,
  // and scale up when rendering.
  m_d3dDevice.GetDeviceCaps( d3dCaps );

  if m_dwTexWidth > d3dCaps.MaxTextureWidth then begin
    m_fTextScale := d3dCaps.MaxTextureWidth / m_dwTexWidth;
    m_dwTexHeight := d3dCaps.MaxTextureWidth;
    m_dwTexWidth := d3dCaps.MaxTextureWidth;
  end;

  // Create a new texture for the font
  Result := m_d3dDevice.CreateTexture( m_dwTexWidth, m_dwTexHeight, 1,
                                      0, D3DFMT_A4R4G4B4,
                                      D3DPOOL_MANAGED, m_Texture );
  if Failed( Result ) then Exit;

  // Prepare to create a bitmap
  ZeroMemory( @bmi.bmiHeader, sizeof( BITMAPINFOHEADER ) );
  with bmi.bmiHeader do begin
    biSize        := sizeof(BITMAPINFOHEADER);
    biWidth       := m_dwTexWidth;
    biHeight      := -m_dwTexHeight;
    biPlanes      := 1;
    biCompression := BI_RGB;
    biBitCount    := 32;
  end;

  // Create a DC and a bitmap for the font
  hDC := CreateCompatibleDC( 0 );
  hbm := CreateDIBSection( hDC, bmi, DIB_RGB_COLORS, Bits, 0, 0 );
  SetMapMode( hDC, MM_TEXT );

  // Create a font.  By specifying ANTIALIASED_QUALITY, we might get an
  // antialiased font, but this is not guaranteed.
  nHeight := MulDiv( m_dwFontHeight, Round( GetDeviceCaps( hDC, LOGPIXELSY ) * m_fTextScale ), 72 );
  if ( m_dwFontFlags and D3DFONT_BOLD ) = D3DFONT_BOLD then dwBold := FW_BOLD else dwBold := FW_NORMAL;
  if ( m_dwFontFlags and D3DFONT_ITALIC ) = D3DFONT_ITALIC then dwItalic := 1 else dwItalic := 0;
  hFont := CreateFont( nHeight, 0, 0, 0, dwBold, dwItalic,
            0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS,
            CLIP_DEFAULT_PRECIS, ANTIALIASED_QUALITY,
            VARIABLE_PITCH, m_strFontName );
  // hm... really need????
  {if hFont = 0 then begin
    Result := E_FAIL;
    Exit;
  end;}

  SelectObject( hDC, hbm );
  SelectObject( hDC, hFont );

  // Set text properties
  SetTextColor( hDC, RGB( 255, 255, 255 ) );
  SetBkColor( hDC, 0 );
  SetTextAlign( hDC, TA_TOP );

  // Loop through all printable character and output them to the bitmap..
  // Meanwhile, keep track of the corresponding tex coords for each character.
  x := 0;
  y := 0;

  for c := 32 to 126 do begin
    str := Chr( c );
    GetTextExtentPoint32( hDC, PChar( str ), 1, nSize );
    if x + DWORD( nSize.cx ) + 1 > m_dwTexWidth then begin
      x := 0;
      inc( y, nSize.cy + 1 );
    end;
    ExtTextOut( hDC, x + 0, y + 0, ETO_OPAQUE, nil, PChar( str ), 1, nil );

    m_fTexCoords[c-32][0] := ( x + 0 ) / m_dwTexWidth;
    m_fTexCoords[c-32][1] := ( y + 0 ) / m_dwTexHeight;
    m_fTexCoords[c-32][2] := ( x + 0 + DWORD( nSize.cx ) ) / m_dwTexWidth;
    m_fTexCoords[c-32][3] := ( y + 0 + DWORD( nSize.cy ) ) / m_dwTexHeight;

    inc( x, nSize.cx + 1 );
  end;

  // Lock the surface and write the alpha values for the set pixels
  Result := m_Texture.LockRect( 0, d3dlr, nil, 0 );
  if Failed( Result ) then goto PEnd;
  i := Integer( d3dlr.pBits );
  j := Integer( Bits );
  for y := 0 to m_dwTexHeight - 1 do
  for x := 0 to m_dwTexWidth - 1 do begin
    pixels := Pointer( j );
    pbits := Pointer( i );
    bAlpha := ( pixels^ and $ff ) shr 4;
    if bAlpha > 0 then
      pbits^ := ( bAlpha shl 12 ) or $0fff
    else
      pbits^ := $0000;
    inc( i, SizeOf( WORD ) );
    inc( j, SizeOf( DWORD ) );
  end;

  // Done updating texture, so clean up used objects
  m_Texture.UnlockRect( 0 );
PEnd:
  DeleteObject( hbm );
  DeleteDC( hDC );
  DeleteObject( hFont );
  Result := S_OK;
end;

//-----------------------------------------------------------------------------
// Name: RestoreDeviceObjects()
// Desc:
//-----------------------------------------------------------------------------
function TD3DFont.RestoreDeviceObjects : HRESULT;
var
  which : WORD;
begin
  // Create vertex buffer for the letters
  Result := m_d3dDevice.CreateVertexBuffer(
                          MAX_NUM_VERTICES * sizeof( FONT2DVERTEX ),
                          D3DUSAGE_WRITEONLY or D3DUSAGE_DYNAMIC, 0,
                          D3DPOOL_DEFAULT, m_VB );
  if Failed( Result ) then Exit;

  // Create the state blocks for rendering text
  for which := 0 to 1 do begin

    m_d3dDevice.BeginStateBlock();
    m_d3dDevice.SetTexture( 0, m_Texture );
    m_d3dDevice.SetRenderState( D3DRS_ALPHABLENDENABLE, 1 );
    m_d3dDevice.SetRenderState( D3DRS_SRCBLEND,   D3DBLEND_SRCALPHA );
    m_d3dDevice.SetRenderState( D3DRS_DESTBLEND,  D3DBLEND_INVSRCALPHA );
    m_d3dDevice.SetRenderState( D3DRS_ALPHATESTENABLE,  1 );
    m_d3dDevice.SetRenderState( D3DRS_ALPHAREF,         $08 );
    m_d3dDevice.SetRenderState( D3DRS_ALPHAFUNC,  D3DCMP_GREATEREQUAL );
    m_d3dDevice.SetRenderState( D3DRS_FILLMODE,   D3DFILL_SOLID );
    m_d3dDevice.SetRenderState( D3DRS_CULLMODE,   D3DCULL_CCW );
    m_d3dDevice.SetRenderState( D3DRS_ZENABLE,          0 );
    m_d3dDevice.SetRenderState( D3DRS_STENCILENABLE,    0 );
    m_d3dDevice.SetRenderState( D3DRS_CLIPPING,         1 );
    m_d3dDevice.SetRenderState( D3DRS_EDGEANTIALIAS,    0 );
    m_d3dDevice.SetRenderState( D3DRS_CLIPPLANEENABLE,  0 );
    m_d3dDevice.SetRenderState( D3DRS_VERTEXBLEND,      0 );
    m_d3dDevice.SetRenderState( D3DRS_INDEXEDVERTEXBLENDENABLE, 0 );
    m_d3dDevice.SetRenderState( D3DRS_FOGENABLE,        0 );
    m_d3dDevice.SetTextureStageState( 0, D3DTSS_COLOROP,   D3DTOP_MODULATE );
    m_d3dDevice.SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
    m_d3dDevice.SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_DIFFUSE );
    m_d3dDevice.SetTextureStageState( 0, D3DTSS_ALPHAOP,   D3DTOP_MODULATE );
    m_d3dDevice.SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE );
    m_d3dDevice.SetTextureStageState( 0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE );
    m_d3dDevice.SetTextureStageState( 0, D3DTSS_MINFILTER, D3DTEXF_POINT );
    m_d3dDevice.SetTextureStageState( 0, D3DTSS_MAGFILTER, D3DTEXF_POINT );
    m_d3dDevice.SetTextureStageState( 0, D3DTSS_MIPFILTER, D3DTEXF_NONE );
    m_d3dDevice.SetTextureStageState( 0, D3DTSS_TEXCOORDINDEX, 0 );
    m_d3dDevice.SetTextureStageState( 0, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_DISABLE );
    m_d3dDevice.SetTextureStageState( 1, D3DTSS_COLOROP,   D3DTOP_DISABLE );
    m_d3dDevice.SetTextureStageState( 1, D3DTSS_ALPHAOP,   D3DTOP_DISABLE );

    if 0 = which then
      m_d3dDevice.EndStateBlock( m_dwSavedStateBlock )
    else
      m_d3dDevice.EndStateBlock( m_dwDrawTextStateBlock );

  end;

  Result := S_OK;
end;

//-----------------------------------------------------------------------------
// Name: InvalidateDeviceObjects()
// Desc: Destroys all device-dependent objects
//-----------------------------------------------------------------------------
function TD3DFont.InvalidateDeviceObjects : HRESULT;
begin
  m_VB := nil;

  // Delete the state blocks
  if m_d3dDevice <> nil then begin
    if m_dwSavedStateBlock > 0 then
      m_d3dDevice.DeleteStateBlock( m_dwSavedStateBlock );
    if m_dwDrawTextStateBlock > 0 then
      m_d3dDevice.DeleteStateBlock( m_dwDrawTextStateBlock );
  end;

  m_dwSavedStateBlock    := 0;
  m_dwDrawTextStateBlock := 0;

  Result := S_OK;
end;

//-----------------------------------------------------------------------------
// Name: DeleteDeviceObjects()
// Desc: Destroys all device-dependent objects
//-----------------------------------------------------------------------------
function TD3DFont.DeleteDeviceObjects : HRESULT;
begin
  m_Texture := nil;
  m_d3dDevice := nil;

  Result := S_OK;
end;

//-----------------------------------------------------------------------------
// Name: GetTextExtent()
// Desc: Get the dimensions of a text string
//-----------------------------------------------------------------------------
function TD3DFont.GetTextExtent( strText : PChar; var nSize : SIZE ) : HRESULT;
var
  tx1, tx2, fRowWidth, fRowHeight, fWidth, fHeight : Real;
  c : Byte;
  sp, sm : Word;
begin
  Result := E_FAIL;
  if nil = strText then Exit;

  fRowWidth  := 0.0;
  fRowHeight := ( m_fTexCoords[0][3] - m_fTexCoords[0][1] ) * m_dwTexHeight;
  fWidth     := 0.0;
  fHeight    := fRowHeight;

  sm := Length( strText );
  sp := 0;

  while sm < sp do begin
    c := Ord( strText[sp] );
    if ( c = $0A ) or ( c = $10 ) then begin
      fRowWidth := 0.0;
      fHeight := fHeight + fRowHeight;
    end;
    if c < 32 then continue;
    tx1 := m_fTexCoords[c-32][0];
    tx2 := m_fTexCoords[c-32][2];

    fRowWidth := fRowWidth + ( tx2 - tx1 ) * m_dwTexWidth;

    if fRowWidth > fWidth then fWidth := fRowWidth;
    inc( sp );
  end;

  nSize.cx := Round( fWidth );
  nSize.cy := Round( fHeight );

  Result := S_OK;
end;

//-----------------------------------------------------------------------------
// Name: DrawTextScaled()
// Desc: Draws scaled 2D text.  Note that x and y are in viewport coordinates
//       (ranging from -1 to +1).  fXScale and fYScale are the size fraction 
//       relative to the entire viewport.  For example, a fXScale of 0.25 is
//       1/8th of the screen width.  This allows you to output text at a fixed
//       fraction of the viewport, even if the screen or window size changes.
//-----------------------------------------------------------------------------
function TD3DFont.DrawTextScaled( x, y, z, fXScale, fYScale : Real;
          dwColor : DWORD; strText : PChar; dwFlags : DWORD ) : HRESULT;
var
  vp : TD3DVIEWPORT8;
  w, h, tx1, ty1, tx2, ty2, sx, sy, sz, rhw, fStartX, fLineHeight : Real;
  dwNumTriangles : DWORD;
  pVertices : ^FONT2DVERTEX;
  c : Byte;
  sp, sm : Word;
begin
  Result := E_FAIL;
  if nil = m_d3dDevice then Exit;

  // Set up renderstate
  m_d3dDevice.CaptureStateBlock( m_dwSavedStateBlock );
  m_d3dDevice.ApplyStateBlock( m_dwDrawTextStateBlock );
  m_d3dDevice.SetVertexShader( D3DFVF_FONT2DVERTEX );
  m_d3dDevice.SetStreamSource( 0, m_VB, sizeof( FONT2DVERTEX ) );

  // Set filter states
  if ( dwFlags and D3DFONT_FILTERED ) = D3DFONT_FILTERED then begin
    m_d3dDevice.SetTextureStageState( 0, D3DTSS_MINFILTER, D3DTEXF_LINEAR );
    m_d3dDevice.SetTextureStageState( 0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR );
  end;
  m_d3dDevice.GetViewport( vp );
  sx  := ( x + 1.0 ) * vp.Width / 2;
  sy  := ( y - 1.0 ) * vp.Height / 2;
  sz  := z;
  rhw := 1.0;
  fStartX := sx;

  fLineHeight := ( m_fTexCoords[0][3] - m_fTexCoords[0][1] ) * m_dwTexHeight;

  // Fill vertex buffer
  dwNumTriangles := 0;
  m_VB.Lock( 0, 0, PByte( pVertices ), D3DLOCK_DISCARD );

  sm := Length( strText );
  sp := 0;

  while sp < sm do begin
    c := Ord( strText[sp] );
    inc( sp );
    if ( c = $0A ) or ( c = $10 ) then begin
      sx := fStartX;
      sy := sy + fYScale * vp.Height;
    end;
    if c < 32 then continue;

    tx1 := m_fTexCoords[c-32][0];
    ty1 := m_fTexCoords[c-32][1];
    tx2 := m_fTexCoords[c-32][2];
    ty2 := m_fTexCoords[c-32][3];

    w := ( tx2 - tx1 ) * m_dwTexWidth;
    h := ( ty2 - ty1 ) * m_dwTexHeight;
    w := w * ( fXScale * vp.Height ) / fLineHeight;
    h := h * ( fYScale * vp.Height ) / fLineHeight;

    pVertices^ := InitFont2DVertex( D3DXVECTOR4
      ( sx + 0 - 0.5, sy + h-0.5, sz, rhw ), dwColor, tx1, ty2 );
    inc( pVertices );
    pVertices^ := InitFont2DVertex( D3DXVECTOR4
      ( sx + 0 - 0.5, sy + 0 - 0.5, sz, rhw ), dwColor, tx1, ty1 );
    inc( pVertices );
    pVertices^ := InitFont2DVertex( D3DXVECTOR4
      ( sx + w - 0.5, sy + h - 0.5, sz, rhw ), dwColor, tx2, ty2 );
    inc( pVertices );
    pVertices^ := InitFont2DVertex( D3DXVECTOR4
      ( sx + w - 0.5, sy + 0 - 0.5, sz, rhw ), dwColor, tx2, ty1 );
    inc( pVertices );
    pVertices^ := InitFont2DVertex( D3DXVECTOR4
      ( sx + w - 0.5, sy + h - 0.5, sz, rhw ), dwColor, tx2, ty2 );
    inc( pVertices );
    pVertices^ := InitFont2DVertex( D3DXVECTOR4
      ( sx + 0 - 0.5, sy + 0 - 0.5, sz, rhw ), dwColor, tx1, ty1 );
    inc( pVertices );
    inc( dwNumTriangles, 2 );

    if dwNumTriangles * 3 > ( MAX_NUM_VERTICES - 6 ) then begin
      // Unlock, render, and relock the vertex buffer
      m_VB.Unlock();
      m_d3dDevice.DrawPrimitive( D3DPT_TRIANGLELIST, 0, dwNumTriangles );
      m_VB.Lock( 0, 0, PByte( pVertices ), D3DLOCK_DISCARD );
      dwNumTriangles := 0;
    end;
    sx := sx + w;
  end;

  // Unlock and render the vertex buffer
  m_VB.Unlock();
  if dwNumTriangles > 0 then
    m_d3dDevice.DrawPrimitive( D3DPT_TRIANGLELIST, 0, dwNumTriangles );

  // Restore the modified renderstates
  m_d3dDevice.ApplyStateBlock( m_dwSavedStateBlock );

  Result := S_OK;
end;

//-----------------------------------------------------------------------------
// Name: DrawText()
// Desc: Draws 2D text
//-----------------------------------------------------------------------------
function TD3DFont.DrawText( sx, sy : Real; dwColor : DWORD;
                      strText : PChar; dwFlags : DWORD ) : HRESULT;
var
  w, h, tx1, ty1, tx2, ty2, fStartX : Real;
  dwNumTriangles : DWORD;
  pVertices : ^FONT2DVERTEX;
  c : Byte;
  sp, sm : Word;
begin
  Result := E_FAIL;
  if nil = m_d3dDevice then Exit;

  // Set up renderstate
  m_d3dDevice.CaptureStateBlock( m_dwSavedStateBlock );
  m_d3dDevice.ApplyStateBlock( m_dwDrawTextStateBlock );
  m_d3dDevice.SetVertexShader( D3DFVF_FONT2DVERTEX );
  m_d3dDevice.SetStreamSource( 0, m_VB, sizeof( FONT2DVERTEX ) );

  // Set filter states
  if ( dwFlags and D3DFONT_FILTERED ) = D3DFONT_FILTERED then begin
    m_d3dDevice.SetTextureStageState( 0, D3DTSS_MINFILTER, D3DTEXF_LINEAR );
    m_d3dDevice.SetTextureStageState( 0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR );
  end;

  fStartX := sx;

  // Fill vertex buffer
  dwNumTriangles := 0;
  m_VB.Lock( 0, 0, PByte( pVertices ), D3DLOCK_DISCARD );

  sm := Length( strText );
  sp := 0;

  while sp < sm do begin
    c := Ord( strText[sp] );
    inc( sp );
    if ( c = $0A ) or ( c = $10 ) then begin
      sx := fStartX;
      sy := sy + ( m_fTexCoords[0][3] - m_fTexCoords[0][1] ) * m_dwTexHeight;
    end;
    if c < 32 then continue;

    tx1 := m_fTexCoords[c-32][0];
    ty1 := m_fTexCoords[c-32][1];
    tx2 := m_fTexCoords[c-32][2];
    ty2 := m_fTexCoords[c-32][3];

    w := ( tx2 - tx1 ) *  m_dwTexWidth / m_fTextScale;
    h := ( ty2 - ty1 ) * m_dwTexHeight / m_fTextScale;

    pVertices^ := InitFont2DVertex( D3DXVECTOR4
      ( sx + 0 - 0.5, sy + h-0.5, 0.9, 1 ), dwColor, tx1, ty2 );
    inc( pVertices );
    pVertices^ := InitFont2DVertex( D3DXVECTOR4
      ( sx + 0 - 0.5, sy + 0 - 0.5, 0.9, 1 ), dwColor, tx1, ty1 );
    inc( pVertices );
    pVertices^ := InitFont2DVertex( D3DXVECTOR4
      ( sx + w - 0.5, sy + h - 0.5, 0.9, 1 ), dwColor, tx2, ty2 );
    inc( pVertices );
    pVertices^ := InitFont2DVertex( D3DXVECTOR4
      ( sx + w - 0.5, sy + 0 - 0.5, 0.9, 1 ), dwColor, tx2, ty1 );
    inc( pVertices );
    pVertices^ := InitFont2DVertex( D3DXVECTOR4
      ( sx + w - 0.5, sy + h - 0.5, 0.9, 1 ), dwColor, tx2, ty2 );
    inc( pVertices );
    pVertices^ := InitFont2DVertex( D3DXVECTOR4
      ( sx + 0 - 0.5, sy + 0 - 0.5, 0.9, 1 ), dwColor, tx1, ty1 );
    inc( pVertices );
    inc( dwNumTriangles, 2 );

    if dwNumTriangles * 3 > ( MAX_NUM_VERTICES - 6 ) then begin
      // Unlock, render, and relock the vertex buffer
      m_VB.Unlock();
      m_d3dDevice.DrawPrimitive( D3DPT_TRIANGLELIST, 0, dwNumTriangles );
      m_VB.Lock( 0, 0, PByte( pVertices ), D3DLOCK_DISCARD );
      dwNumTriangles := 0;
    end;

    sx := sx + w;
  end;

  // Unlock and render the vertex buffer
  m_VB.Unlock();
  if dwNumTriangles > 0 then
    m_d3dDevice.DrawPrimitive( D3DPT_TRIANGLELIST, 0, dwNumTriangles );

  // Restore the modified renderstates
  m_d3dDevice.ApplyStateBlock( m_dwSavedStateBlock );

  Result := S_OK;
end;

//-----------------------------------------------------------------------------
// Name: DrawText()
// Desc: Draws 2D text
//-----------------------------------------------------------------------------
function TD3DFont.Render3DText( strText : PChar; dwFlags : DWORD ) : HRESULT;
var
  x, y, w, h, tx1, ty1, tx2, ty2, fStartX : Real;
  dwNumTriangles : DWORD;
  pVertices : ^FONT3DVERTEX;
  c : Byte;
  sp, sm : Word;
  nSize : SIZE;
begin
  Result := E_FAIL;
  if nil = m_d3dDevice then Exit;

  // Set up renderstate
  m_d3dDevice.CaptureStateBlock( m_dwSavedStateBlock );
  m_d3dDevice.ApplyStateBlock( m_dwDrawTextStateBlock );
  m_d3dDevice.SetVertexShader( D3DFVF_FONT3DVERTEX );
  m_d3dDevice.SetStreamSource( 0, m_VB, sizeof( FONT2DVERTEX ) );

  // Set filter states
  if ( dwFlags and D3DFONT_FILTERED ) = D3DFONT_FILTERED then begin
    m_d3dDevice.SetTextureStageState( 0, D3DTSS_MINFILTER, D3DTEXF_LINEAR );
    m_d3dDevice.SetTextureStageState( 0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR );
  end;

  x := 0;
  y := 0;

  // Center the text block at the origin
  if ( dwFlags and D3DFONT_CENTERED ) = D3DFONT_CENTERED then begin
    GetTextExtent( strText, nSize );
    x := -( ( nSize.cx ) / 10.0 ) / 2.0;
    y := -( ( nSize.cy ) / 10.0 ) / 2.0;
  end;
  
  // Turn off culling for two-sided text
  if( dwFlags and D3DFONT_TWOSIDED ) = D3DFONT_TWOSIDED then
    m_d3dDevice.SetRenderState( D3DRS_CULLMODE, D3DCULL_NONE );
  fStartX := x;

  // Fill vertex buffer
  dwNumTriangles := 0;
  m_VB.Lock( 0, 0, PByte( pVertices ), D3DLOCK_DISCARD );

  sm := Length( strText );
  sp := 0;

  while sp < sm do begin
    c := Ord( strText[sp] );
    inc( sp );
    if ( c = $0A ) or ( c = $10 ) then begin
      x := fStartX;
      y := y - ( m_fTexCoords[0][3] - m_fTexCoords[0][1] ) * m_dwTexHeight / 10.0;
    end;
    if c < 32 then continue;

    tx1 := m_fTexCoords[c-32][0];
    ty1 := m_fTexCoords[c-32][1];
    tx2 := m_fTexCoords[c-32][2];
    ty2 := m_fTexCoords[c-32][3];

    w := ( tx2 - tx1 ) * m_dwTexWidth  / ( 10.0 * m_fTextScale );
    h := ( ty2 - ty1 ) * m_dwTexHeight / ( 10.0 * m_fTextScale );

    pVertices^ := InitFont3DVertex(
      D3DXVECTOR3( x + 0, y + 0, 0 ), D3DXVECTOR3( 0, 0, -1 ), tx1, ty2 );
    inc( pVertices );
    pVertices^ := InitFont3DVertex(
      D3DXVECTOR3( x + 0, y + h, 0 ), D3DXVECTOR3( 0, 0, - 1 ), tx1, ty1 );
    inc( pVertices );
    pVertices^ := InitFont3DVertex(
      D3DXVECTOR3( x + w, y + 0, 0 ), D3DXVECTOR3( 0, 0, -1 ), tx2, ty2 );
    inc( pVertices );
    pVertices^ := InitFont3DVertex(
      D3DXVECTOR3( x + w, y + h, 0 ), D3DXVECTOR3( 0, 0, -1 ), tx2, ty1 );
    inc( pVertices );
    pVertices^ := InitFont3DVertex(
      D3DXVECTOR3( x + w, y + 0, 0 ), D3DXVECTOR3( 0, 0, -1 ), tx2, ty2 );
    inc( pVertices );
    pVertices^ := InitFont3DVertex(
      D3DXVECTOR3( x + 0, y + h, 0 ), D3DXVECTOR3( 0, 0, -1 ), tx1, ty1 );
    inc( pVertices );
    inc( dwNumTriangles, 2 );

    if dwNumTriangles * 3 > ( MAX_NUM_VERTICES - 6 ) then begin
      // Unlock, render, and relock the vertex buffer
      m_VB.Unlock();
      m_d3dDevice.DrawPrimitive( D3DPT_TRIANGLELIST, 0, dwNumTriangles );
      m_VB.Lock( 0, 0, PByte( pVertices ), D3DLOCK_DISCARD );
      dwNumTriangles := 0;
    end;

    x := x + w;
  end;

  // Unlock and render the vertex buffer
  m_VB.Unlock();
  if dwNumTriangles > 0 then
    m_d3dDevice.DrawPrimitive( D3DPT_TRIANGLELIST, 0, dwNumTriangles );

  // Restore the modified renderstates
  m_d3dDevice.ApplyStateBlock( m_dwSavedStateBlock );

  Result := S_OK;
end;

end.
