unit Keyboard;

interface

uses
  Windows;

type

   TVirtualKeyCode = Integer;

function IsKeyDown(c : Char) : Boolean; overload;
function IsKeyDown(vk : TVirtualKeyCode) : Boolean; overload;
function KeyPressed(minVkCode : TVirtualKeyCode = 0) : TVirtualKeyCode;
function VirtualKeyCodeToKeyName(vk : TVirtualKeyCode) : String;
function KeyNameToVirtualKeyCode(const keyName : String) : TVirtualKeyCode;
function CharToVirtualKeyCode(c : Char) : TVirtualKeyCode;

implementation

uses SysUtils;

const
   cLBUTTON = 'LBUTTON';
   cMBUTTON = 'MBUTTON';
   cRBUTTON = 'RBUTTON';

   cLEFT=     'Left Arrow';
   cRIGHT=    'Right Arrow';
   cUP=       'Up Arrow';
   cDOWN=     'Down Arrow';
   cINSERT=   'Insert';
   cDELETE=   'Delete';
   cHOME=     'Home';
   cEND=      'End';
   cPGUP=     'Page Up';
   cPGDOWN=   'Page Down';




function IsKeyDown(c : Char) : Boolean;
var
   vk : Integer;
begin
   vk:=VkKeyScan(c) and $FF;
   if vk<>$FF then
      Result:=(GetAsyncKeyState(vk)<0)
   else Result:=False;
end;

function IsKeyDown(vk : TVirtualKeyCode) : Boolean;
begin
   Result:=(GetAsyncKeyState(vk)<>0);
end;

function KeyPressed(minVkCode : TVirtualKeyCode = 0) : TVirtualKeyCode;
var
   i : Integer;
   buf : TKeyboardState;
begin
   Assert(minVkCode>=0);
   Result:=-1;
   if GetKeyboardState(buf) then begin
      for i:=minVkCode to High(buf) do begin
         if (buf[i] and $80)<>0 then begin
            Result:=i;
            Break;
         end;
      end;
   end;
end;

function VirtualKeyCodeToKeyName(vk : TVirtualKeyCode) : String;
var
   nSize : Integer;
begin
   case vk of
      VK_LBUTTON : Result:=cLBUTTON;
      VK_MBUTTON : Result:=cMBUTTON;
      VK_RBUTTON : Result:=cRBUTTON;
      VK_LEFT:Result:=cLeft;
      VK_RIGHT:Result:=cRight;
      VK_UP:Result:=cUp;
      VK_DOWN:Result:=cDown;
      VK_Insert:Result:=cInsert;
      VK_Delete:Result:=cDelete;
      VK_Home:Result:=cHome;
      VK_End:Result:=cEnd;
      VK_Prior:Result:=cPGUP;
      VK_Next:Result:=cPGDOWN;


   else
      nSize:=32;
      SetLength(Result, nSize);
      vk:=MapVirtualKey(vk, 0);
      nSize:=GetKeyNameText((vk and $FF) shl 16, PChar(Result), nSize);
      SetLength(Result, nSize);
   end;
end;

function KeyNameToVirtualKeyCode(const keyName : String) : TVirtualKeyCode;
var
   i : Integer;
begin
   if keyName=cLBUTTON then
      Result:=VK_LBUTTON
   else if keyName=cMBUTTON then
      Result:=VK_MBUTTON
   else if keyName=cRBUTTON then
      Result:=VK_RBUTTON
   else if keyName=cLeft then
      Result:=VK_LEFT
   else if keyName=cRIGHT then
      Result:=VK_RIGHT
   else if keyName=cUP then
      Result:=VK_UP
   else if keyName=cDOWN then
      Result:=VK_DOWN

   else begin
      Result:=-1;
      for i:=0 to 255 do begin
         if CompareText(VirtualKeyCodeToKeyName(i), keyName)=0 then begin
            Result:=i;
            Break;
         end;
      end;
   end;
end;

function CharToVirtualKeyCode(c : Char) : TVirtualKeyCode;
begin
   Result:=VkKeyScan(c) and $FF;
   if Result=$FF then Result:=-1;
end;

end.
