unit MIDList;
////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
///      VAMPYRE   SOFTWORKS    
////////////////////////////////////////////////////////////////
///      this source is free for noncomercional use 
///
///      [Developer] : Mar
///      last update 4.5.2001  
////////////////////////////////////////////////////////////////

interface

uses
  Windows, Messages, SysUtils, Classes, 
  Controls,  Hash{,DsgnIntF};

type

  TMID = class(TPersistent)
  private
  protected
    procedure DefineProperties(Filer: TFiler); override;
    procedure ReadData(Stream: TStream); virtual;
    procedure WriteData(Stream: TStream); virtual;
  public
    MS:TMemoryStream;
    constructor Create;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    procedure Clear;
    procedure LoadFromFile(const FileName : string);
    procedure LoadFromStream(Stream: TStream);
    procedure SaveToFile(const FileName : string);
    procedure SaveToStream(Stream: TStream);
  published
  end;


  TMIDCollection = class;

  TMIDCollectionItem = class(THashCollectionItem)
  private
    FMID: TMID;
    function GetMIdCollection: TMIDCollection;
    procedure SetMID(Value: TMID);
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    property MIDCollection: TMIDCollection read GetMIDCollection;
  published
    property MID: TMID read FMID write SetMId;
  end;

  TMIDCollection = class(THashCollection)
  private
    FOwner: TPersistent;
    function GetItem(Index: Integer): TMIDCollectionItem;
  protected
    function GetOwner: TPersistent; override;
  public
    constructor Create(AOwner: TPersistent);
    function Find(const Name: string): TMIDCollectionItem;
    procedure LoadFromFile(const FileName: string);
    procedure LoadFromStream(Stream: TStream);
    procedure SaveToFile(const FileName: string);
    procedure SaveToStream(Stream: TStream);
    property Items[Index: Integer]: TMIDCollectionItem read GetItem; default;
  end;




  TCustomMIDList = class(TComponent)
  private
    FItems: TMIDCollection;
    procedure SetItems(Value: TMIDCollection);
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Items: TMIDCollection read FItems write SetItems;
  end;


  TMIDList = class(TCustomMIDList)
  published
    property Items;
  end;


{  TMIDProperty = class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetValue: string; override;
    function GetAttributes: TPropertyAttributes; override;
  end;
 }

const MIDProp='MIDIfile';
      MIDRes='VAMPYRE MIDI COLLECTION';


implementation

//uses Mideditu;


constructor TMID.Create;
 begin
  inherited create;
  MS:=TMemoryStream.Create;
 end;

destructor TMID.Destroy;
begin
  MS.Free;
  inherited Destroy;
end;

procedure TMID.Assign(Source: TPersistent);
var
  AMID: TMID;
begin
  if Source=nil then
  begin
    Clear;
  end else if Source is TMID then
  begin
    if Source<>Self then
    begin
      AMID := TMID(Source);
      MS.LoadFromStream(AMID.MS);
    end;
  end else
    inherited Assign(Source);
end;

procedure TMID.Clear;
begin
  MS.Clear;
end;

procedure TMID.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineBinaryProperty(MIDProp, ReadData, WriteData, True);
end;

procedure TMID.LoadFromFile(const FileName : string);
var
  Stream: TFileStream;
begin
  Stream := TFileStream.Create(FileName, fmOpenRead or fmShareDenyWrite);
  try
    LoadFromStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure TMID.LoadFromStream(Stream: TStream);
begin
  Clear;
  MS.LoadfromStream(Stream);
end;

procedure TMID.ReadData(Stream: TStream);
begin
  LoadFromStream(Stream);
end;

procedure TMID.SaveToFile(const FileName : string);
var
  Stream: TFileStream;
begin
  Stream := TFileStream.Create(FileName, fmCreate);
  try
    SaveToStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure TMID.SaveToStream(Stream: TStream);
begin
     MS.SaveToStream(Stream);
end;


procedure TMID.WriteData(Stream: TStream);
begin
  SaveToStream(Stream);
end;






constructor TMIDCollectionItem.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  FMID := TMID.Create;
  end;

destructor TMIDCollectionItem.Destroy;
begin
  FMID.Free;
  inherited Destroy;
end;

procedure TMIDCollectionItem.Assign(Source: TPersistent);
begin
  if Source is TMIDCollectionItem then
  begin
    Name := TMIDCollectionItem(Source).Name;
    FMID.Assign(TMIDCollectionItem(Source).FMID);

  end else
    inherited Assign(Source);
end;


function TMIDCollectionItem.GetMIDCollection: TMIDCollection;
begin
  Result := Collection as TMIDCollection;
end;


procedure TMIDCollectionItem.SetMID(Value: TMID);
begin
  FMID.Assign(Value);
end;

{  TMIDCollection  }

constructor TMIDCollection.Create(AOwner: TPersistent);
begin
  inherited Create(TMIDCollectionItem);
  FOwner := AOwner;
end;

function TMIDCollection.GetItem(Index: Integer): TMIDCollectionItem;
begin
  Result := TMIDCollectionItem(inherited Items[Index]);
end;

function TMIDCollection.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

function TMIDCollection.Find(const Name: string): TMIDCollectionItem;
var
  i: Integer;
begin
  Result:=nil;
  i := Indexof(Name);
  if i=-1 then
   Result:=nil else
  Result := Items[i];
end;

type
  TMIDCollectionComponent = class(TComponent)
  private
    FList: TMIDCollection;
  published
    property List: TMIDCollection read FList write FList;
  end;

procedure TMIDCollection.LoadFromFile(const FileName: string);
var
  Stream: TFileStream;
begin
  Stream := TFileStream.Create(FileName, fmOpenRead or fmShareDenyWrite);
  try
    LoadFromStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure TMIDCollection.LoadFromStream(Stream: TStream);
var
  Component: TMIDCollectionComponent;
begin
  Clear;
  Component := TMIDCollectionComponent.Create(nil);
  try
    Component.FList := Self;
    Stream.ReadComponentRes(Component);

  finally
    Component.Free;
  end;
end;

procedure TMIDCollection.SaveToFile(const FileName: string);
var
  Stream: TFileStream;
begin
  Stream := TFileStream.Create(FileName, fmCreate);
  try
    SaveToStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure TMIDCollection.SaveToStream(Stream: TStream);
var
  Component: TMIDCollectionComponent;
begin
  Component := TMIDCollectionComponent.Create(nil);
  try
    Component.FList := Self;
    Stream.WriteComponentRes(MIDRes,Component);
  finally
    Component.Free;
  end;
end;

{  TCustomDXMIDList  }

constructor TCustomMIDList.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FItems := TMIDCollection.Create(Self);
end;

destructor TCustomMIDList.Destroy;
begin
  FItems.Free;
  inherited Destroy;
end;


procedure TCustomMIDList.SetItems(Value: TMIDCollection);
begin
  FItems.Assign(Value);
end;


{procedure TMIDProperty.Edit;
var
  Form: TMIDEdit;
begin
  Form := TMIDEdit.Create(nil);
  try
    Form.MID := TMID(GetOrdValue);
    Form.ShowModal;
    if Form.Tag <> 0 then
    begin
      SetOrdValue(Integer(Form.MID));

      Designer.Modified;
    end;
  finally
    Form.Free;
  end;
  end;

function TMIDProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

function TMIDProperty.GetValue: string;
begin
  if TMID(GetOrdValue).MS.Size < 1 then
    Result := '(NONE)'

  else
    Result := Format('(%s)', [TObject(GetOrdValue).ClassName]);
end;



 }







end.
