unit MIDPlay;
////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////
///      VAMPYRE   SOFTWORKS
////////////////////////////////////////////////////////////////
///      this source is free for noncomercional use
///
///      [Developer] : Marek Mauder , pentar@seznam.cz
///      started     : 1.5.2001
///      last update : 17.6.2001
///           narychlo changed  09.02.2002
////////////////////////////////////////////////////////////////
//        TMIDPlay component provides:
//         -Playing standard midi files trough DIRECT MUSIC
//         -Loading midi files from files,streams,memory and TMIDList
//         -Changing DIRECT MUSIC ports
//         -Playlist support
//
//
//
/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////
//        Vampyre SoftWorks are :
//                            Marek "MAR" Mauder
//                            Karel "KaKL Jr III" Kl
//////////////////////////////////////////////////////////////////




interface

uses

  Windows, SysUtils, Classes, Graphics,
  Controls, gameTypes,WinFunc,Forms,
  ComObj, DirectMusic, MIDList, VTextList, Contnrs
  ;

type

  IDMPerf = IDirectMusicPerformance;
  IDMSeg = IDirectMusicSegment;
  IDMLoader = IDirectMusicLoader;
  IDMSegState = IDirectMusicSegmentState;
  IDMusic = IDirectMusic;
  IDMPort = IDirectMusicPort;
  IDMObj = IDirectMusicObject;




  TMIDPlay = class;

  TSegList = class;

  TMIDPlayList = class;

  TPlayer = (plMIDPlay, plPlayList, plSegment);

  TPlayThread = class(TThread)
  public
    Play: TMIDPlay;
    List: TMIDPlayList;
    Player: TPlayer;
    constructor Create(Ply: TMIDPlay; PL: TMIDPlayList);
    destructor Destroy; override;
    procedure Start(Mode: TPlayer);
    procedure Execute; override;

  end;

  TSegment = class(TPersistent)
  protected
    function GetPlaying: boolean;
    function GetLength: long;
    function GetPos: long;
    procedure SetPos(Value: long);
    procedure SetLength(Value: long);

  public
    MIDPlay: TMIDPlay;
    Seg: IDMSeg;
    State: IDMSegState;

    Loaded: boolean;
    constructor Create(Play: TMIDPlay);
    destructor Destroy; override;
    procedure LoadFromFile(fn: string; Ply: boolean = false);
    procedure LoadFromMemory(data: Pointer; size: dword; Ply: boolean = false);
    procedure LoadFromStream(S: TStream; Ply: boolean = false);
    procedure UnLoad;
    procedure Play;
    procedure Pause;
    procedure Stop;

    property Playing: boolean read GetPlaying;
    property Position: long read GetPos write SetPos;
    property Length: long read GetLength write SetLength;


  end;


  TPListItem = class
  public
    Played: boolean;
    Name: string;
    Theme: string;
    Seg: TSegment;

    destructor Destroy;override;
  end;

  TPLOptFlags = (poShuffle, poContinuous);

  TPLOptions = set of TPLOptFlags;

  TLoadFrom = (lfFiles, lfList);

  TMIDPlayList = class(TPersistent)
  private
    FList: TObjectList;
    FMIDPlay: TMIDPlay;
    FMIDList: TMIDList;
    FOptions: TPLOptions;
    FStrList: TStringList;
    FStoreList: TMIDList;

    ASeg: TSegment;
    AItem: TPlistItem;
  protected

  public
    ItemIdx: long;

    constructor Create(Ply: TMIDPlay);
    destructor Destroy; override;

    procedure PlayNext;
    procedure PlayItem(idx: long);
    procedure Play;
    procedure Pause;
    procedure Restart(Ply: boolean = false);
    procedure LoadFromList(Pl: TStrings; Ply: boolean = false);
    procedure LoadFromCustomList(Pl: TStrings; List: TMIDList; Ply: boolean = false);
    procedure LoadFromFiles(Pl: TStrings; Ply: boolean = false);
    procedure LoadFromItem(Item: TTextCollectionItem; From: TLoadFrom; Ply: boolean = false);


    property MIDPlay: TMIDPlay read FMIDPlay write FMIDPlay;
    property List: TObjectList read FList;
    property MIDList: TMIDList read FMIDList;
    property StrList: TStringList read FStrList;
  published
    property Options: TPlOptions read FOptions write FOptions;
    property StoreList: TMIDList read FMIDList write FMIDList;

  end;

  TMidOptFlags = (moAutoInit, moAutoPlay, moUseDXSound, moLoop);

  TMIDOptions = set of TMIDOptFlags;

  TEndOfMIDI = procedure(Sender: TObject; Segment: TSegment) of object;

  TPort = class
  private
    FMIDPlay: TMIDPlay;
    FPort: IDMPort;
    FIndex: long;
    FName:string;

  protected
    function GetPortCount: long;
    procedure SetPortIdx(idx: long);
    procedure SetPortName(Name: string);

  public
    constructor Create(Ply: TMIDPlay);
    destructor Destroy; override;

    function GetPortName(idx: long): string;
    function GetPortCaps(idx: long): TDMUS_PORTCAPS;
    function GetNameIndex(PName:string):long;

    property PortCount: long read GetPortCount;
    property Port: IDMPort read FPort;
    property PortIndex: long read FIndex write SetPortIdx;
    property PortName: string read FName write SetPortName;
    property MIDPlay:TMIDPlay read FMIDPlay write FMIDPlay;

  published

  end;







  TMIDPlay = class(TComponent)
  private
    Fperf: IDMPerf;
    FMusic: IDMusic;
    FLoader: IDMLoader;

//    FSound: TCustomDXSound;
    FOptions: TMIDOptions;

    FFileName: TFileName;
    FReady: boolean;

    FDefSeg: TSegment;

    FList: TSegList;
    FPlayList: TMIDPlayList;

    FMIDIEnd: TEndOfMIDI;

    FThread: TPlayThread;

    FPort: TPort;
    { Private declarations }

//    procedure DXSNotify(Sender: TCustomDXSound; NotifyType: TDXSoundNotifyType);
  protected
//    procedure SetSound(Value: TCustomDXSound);
    procedure Loaded; override;
    function GetPlaying: boolean;
    function GetPlayingAll: boolean;
    function GetTempo: dword;
    procedure SetTempo(Value: dword);
    function GetVolume: long;
    procedure SetVolume(Value: long);
    procedure SetFile(Value:TFilename);

    procedure Update;

    { Protected declarations }
  public
    ASeg: TSegment;

    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

    procedure Init;
    procedure Play;
    procedure Stop;
    procedure Pause;
    procedure StopAll;

    property Ready: boolean read FReady;
    property SegList: TSegList read FList;
    property Playing: boolean read GetPlaying;
    property PlayingAll: boolean read GetPlayingAll;
    property Segment: TSegment read FDefSeg write FDefSeg;
    property Port: TPort read FPort;

    property Performance: IDMPerf read Fperf;
    property DMusic: IDMusic read FMusic;
    property Loader: IDMLoader read FLoader;

    property Tempo: dword read GetTempo write SetTempo;
    property Volume: long read GetVolume write SetVolume;

    property Thread: TPlayThread read FThread;
    { Public declarations }
  published

//    property DXSound: TCustomDXSound read FSound write SetSound;
    property Options: TMIDOptions read FOptions write FOptions;
    property FileName: TFilename read FFileName write SetFile;
    property PlayList: TMIDPlayList read FPlayList write FPlayList;
    property EndOfMIDI: TEndOfMIDI read FMIDiEnd write FMIDiEnd;
    { Published declarations }
  end;

  TSegList = class(TList)
  private
    FOwnsObjects: Boolean;
  protected
    procedure Notify(Ptr: Pointer; Action: TListNotification); override;
    function GetItem(Index: Integer): TSegment;
    procedure SetItem(Index: Integer; ASeg: TSegment);
  public
    constructor Create; overload;
    constructor Create(AOwnsObjects: Boolean); overload;

    function Add(ASeg: TSegment): TSegment;
    function Remove(ASeg: TSegment): Integer;
    function IndexOf(ASeg: TSegment): Integer;
    procedure Insert(Index: Integer; ASeg: TSegment);
    property OwnsObjects: Boolean read FOwnsObjects write FOwnsObjects;
    property Items[Index: Integer]: TSegment read GetItem write SetItem; default;
  end;


const
     // tempo in percents
  MASTERTEMPO_MIN = 1;
  MASTERTEMPO_MAX = 10000;

     // volume in decibells
  MASTERVOL_MIN = -50;
  MASTERVOL_MAX = 50;


procedure Register;

implementation


////////////////////////// TSEGMENT METHODS ///////////////////

constructor TSegment.Create(Play: TMIDPlay);
begin
  inherited Create;
  MIDPlay := Play;

end;

destructor TSegment.Destroy;
begin

  Unload;
  Seg := nil;
  State := nil;
  inherited Destroy;
end;

procedure TSegment.LoadFromFile(fn: string; Ply: boolean);
var
  Obj: TDMUS_OBJECTDESC;
begin
  if MIDPlay.FLoader = nil then Exit;
  if Seg <> nil then
    Seg := nil;
  Zeromemory(@Obj, SizeOf(TDMUS_OBJECTDESC));
  with Obj do
  begin
    dwsize := SizeOf(TDMUS_OBJECTDESC);
    guidclass := CLSID_DirectMusicSegment;
    StringToWideChar(fn, wszFilename, sizeof(wszFilename));
    dwvaliddata := DMUS_OBJ_CLASS or DMUS_OBJ_FULLPATH or DMUS_OBJ_FILENAME;
  end;
  MIDPlay.FLoader.GetObject(Obj, IID_IDirectMusicSegment, Pointer(Seg));
  Seg.setParam(GUID_StandardMidiFile, $FFFFFFFF, 0, 0, Pointer(MIDPlay.FPerf));
  Seg.setParam(GUID_Download, $FFFFFFFF, 0, 0, Pointer(MIDPlay.FPerf));
  Loaded := true;
  if Ply then Self.Play;
end;

procedure TSegment.LoadFromMemory(data: Pointer; size: dword; Ply: boolean);
var
  Obj: TDMUS_OBJECTDESC;
begin
  if MIDPlay.FLoader = nil then Exit;
  if Seg <> nil then
    Seg := nil;
  Zeromemory(@Obj, SizeOf(TDMUS_OBJECTDESC));
  with Obj do
  begin
    dwsize := SizeOf(TDMUS_OBJECTDESC);
    guidclass := CLSID_DirectMusicSegment;
    llMemlength := size;
    pbmemData := data;
    dwvaliddata := DMUS_OBJ_CLASS or DMUS_OBJ_MEMORY;
  end;
  MIDPlay.FLoader.GetObject(Obj, IID_IDirectMusicSegment, Pointer(Seg));
  Seg.setParam(GUID_StandardMidiFile, $FFFFFFFF, 0, 0, Pointer(MIDPlay.FPerf));
  //Seg.setParam(GUID_Download, $FFFFFFFF, 0, 0, Pointer(MIDPlay.FPerf));
  Loaded := true;
  if Ply then Self.Play;
end;

procedure TSegment.LoadFromStream(S: TStream; Ply: boolean);
var
  Data: Pointer;
  Obj: TDMUS_OBJECTDESC;
begin
  if MIDPlay.FLoader = nil then Exit;
  if s.size = 0 then Exit;


  Data:=Allocmem(s.size);
  s.position := 0;
  s.ReadBuffer(Data^, s.size);

   if Seg <> nil then
    Seg := nil;
  Zeromemory(@Obj, SizeOf(TDMUS_OBJECTDESC));
  with Obj do
  begin
    dwsize := SizeOf(TDMUS_OBJECTDESC);
    guidclass := CLSID_DirectMusicSegment;
    llMemlength := s.size;
    pbmemData := data;
    dwvaliddata := DMUS_OBJ_CLASS or DMUS_OBJ_MEMORY;
  end;
  MIDPlay.FLoader.GetObject(Obj, IID_IDirectMusicSegment, Pointer(Seg));
  Seg.setParam(GUID_StandardMidiFile, $FFFFFFFF, 0, 0, Pointer(MIDPlay.FPerf));
  Seg.setParam(GUID_Download, $FFFFFFFF, 0, 0, Pointer(MIDPlay.FPerf));
  FreeMem(Data);
  Loaded := true;
  if Ply then Self.Play;
end;

procedure TSegment.Play;
begin
  if (Seg <> nil) and (MIDPlay.Ready) then
  begin
    MIDPlay.Fperf.PlaySegment (seg, DMUS_PLAYMODE_FIXED, 0, @State);
    MIDPlay.ASeg := Self;
    MIDPlay.Thread.Start(plSegment);
  end;
end;

procedure TSegment.Unload;
begin
  if Seg = nil then Exit;
  Seg.setParam(GUID_Unload, $FFFFFFFF, 0, 0, Pointer(MIDPlay.FPerf));
  Seg := nil;
  State := nil;
  Loaded := false;
end;

procedure TSegment.Stop;
begin
  if MIDPlay=nil then Exit;
  if (Seg <> nil) and (MIDPlay.Ready) then
  begin
    Seg.SetStartPoint(0);
    MIDPlay.Fperf.Stop(Seg, State, 0, 0);

    MIDPlay.Thread.Suspended:=true;
  end;
end;

procedure TSegment.Pause;
var
  Pos: long;
begin
  if (Seg <> nil)and(State<>nil) and (MIDPlay.Ready) then
  begin
    State.GetSeek(Pos);
    MIDPlay.Fperf.Stop(Seg, State, 0, 0);
    Seg.SetStartPoint(Pos);
    MIDPlay.Thread.Suspended:=true;
  end;
end;

function TSegment.GetPlaying: boolean;
begin
  if (MIDPlay.Fperf <> nil) and (MIDPlay.Ready) then
    Result := (MIDPlay.FPerf.IsPlaying(Seg, State) = S_OK)
  else
    Result := false;

end;

function TSegment.GetLength: long;
begin
  if Seg <> nil then Seg.GetLength(Result);

end;

function TSegment.GetPos: long;
begin
  if State <> nil then State.GetSeek(Result);
end;

procedure TSegment.SetPos(Value: long);
begin
  if Seg <> nil then
  begin
    Seg.SetStartPoint(Value + 1);
    if Playing then Play;
  end;
end;

procedure TSegment.SetLength(Value: long);
begin
  if Seg <> nil then
  begin
    Seg.SetLength(Value);
    if Playing and (Position < Value) then
    begin pause; play;
    end;
  end;

end;


 ////////////////////////// TMIDPLAY METHODS ///////////////////


constructor TMIDPlay.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FOptions := [moAutoInit, moAutoPlay];
  FList := TSegList.Create;
  FPlayList := TMIDPlayList.Create(Self);
  FPort:=TPort.Create(Self);
  FThread := TPlayThread.Create(Self, Playlist);
  FDefSeg := TSegment.Create(Self);


end;

destructor TMidPlay.Destroy;
begin
  StopAll;
  PlayList.Free;
  SegList.Free;
  Segment.Free;
  FPort.Free;
  if FPerf <> nil then FPerf.Closedown;
  FPerf := nil;
  FLoader := nil;
  FMusic := nil;
  inherited Destroy;
end;

procedure TMIDPlay.Init;
var
 // DS: IDirectSound;
  Handle: long;
  Param: Longbool;
begin
  FReady := false;
  if PlayingAll then StopAll;
  if FPerf <> nil then FPerf.CloseDown;
  Fperf := nil;
  FLoader := nil;
  FMusic := nil;

  try
 {   if moUseDXsound in Options then
      DS := DxSound.DSound.ISound
    else
      DS := nil;
  }  Handle := TForm(Owner).Handle;
    FMusic := IDMusic(CreateComObject(CLSID_DIRECTMUSIC));
    FMusic.SetDirectSound({DS}nil, Handle);
    FPerf := IDMPerf(CreateComObject(CLSID_DIRECTMUSICPERFORMANCE));
    FPerf.Init(FMusic, {DS}nil, Handle);
    Param := true;
    FPerf.SetGlobalParam(GUID_PerfAutoDownload, @Param, SizeOf(Param));
    FPerf.AddPort(nil);
    FLoader := IDMLoader(CreateComObject(CLSID_DIRECTMUSICLOADER));
    FReady := true;
  except

  end;

end;

{procedure TMIDPlay.DXSNotify(Sender: TCustomDXSound; NotifyType: TDXSoundNotifyType);
begin
  case Notifytype of
    dsntDestroying: DXSound := nil;
    dsntInitialize:
      if (not Ready) and (moAutoInit in Options)
        and (moUseDxSound in Options) then
      begin
        Init; if (moAutoPlay in Options) then Play;
      end;


  end;
end;
 }
{procedure TMIDPlay.SetSound(Value: TCustomDXSound);
begin
  if FSound <> nil then
    FSound.UnRegisterNotifyEvent(DXSNotify);

  FSound := Value;

  if FSound <> nil then
    FSound.RegisterNotifyEvent(DXSNotify);

end;
 }
procedure TMIDPlay.Loaded;
begin
  if csDesigning in ComponentState then Exit;
  if (not (moUseDXSound in Options)) then
  begin
    if moAutoInit in Options then
    begin
      Init;
      if (moAutoPlay in Options) then
        Play;
    end;
  end;

end;

procedure TMIDPlay.SetFile(Value:TFilename);
 begin
  FFileName:=Value;
  Play;
 end;

procedure TMIDPlay.Play;
begin
  if not Segment.Loaded then
    if FileExists(FFilename) then
      Segment.LoadFromfile(FFilename)
    else if FileExists(ExeDir + FFilename) then
      Segment.LoadFromfile(ExeDir + FFilename)
    else
      ;
  Segment.Play;
  Thread.Start(plMIDPlay);

end;

procedure TMIDPlay.Stop;
begin
  Segment.Stop;
end;

procedure TMIDPlay.Pause;
begin
  Segment.Pause;
end;

function TMIDPlay.GetPlaying: boolean;
begin
  Result := Segment.Playing;
end;

function TMIDPlay.GetPlayingAll: boolean;
begin
  if ASeg = nil then
    Result := false
  else
    Result := ASeg.Playing;
end;


function TMIDPlay.GetTempo: dword;
var
  t: single;
begin
  if not Ready then Exit;
  FPerf.GetGlobalParam(GUID_PERFMASTERTEMPO, @t, sizeof(t));
  Result := Round(t * 100);

end;

procedure TMIDPlay.SetTempo(Value: dword);
var
  t: single;
begin
  if not Ready then Exit;
  if VALUE > MASTERTEMPO_MAX then VALUE := MASTERTEMPO_MAX;
  if VALUE < MASTERTEMPO_MIN then VALUE := MASTERTEMPO_MIN;
  t := value / 100;
  FPerf.SetGlobalParam(GUID_PERFMASTERTEMPO, @t, sizeof(t));


end;


function TMiDPlay.GetVolume: long;
var
  v: long;
begin
  if not Ready then Exit;
  FPerf.GetGlobalParam(GUID_PERFMASTERVOLUME, @v, sizeof(v));
  Result := v;

end;

procedure TMiDPlay.SetVolume(Value: long);
var
  v: long;
begin
  if not Ready then Exit;
  if VALUE > MASTERVOL_MAX then VALUE := MASTERVOL_MAX;
  if VALUE < MASTERVOL_MIN then VALUE := MASTERVOL_MIN;
  v := (Value * 100) - 100;
  FPerf.SetGlobalParam(GUID_PERFMASTERVOLUME, @v, sizeof(v));

end;

procedure TMIDPlay.StopAll;
begin
  if ASeg <> nil then
    ASeg.Stop;
end;

procedure TMIDPlay.Update;
begin
  if ASeg.Position >= ASeg.Length then
  begin
    if Assigned(FMIDiEnd) then FMIDIEnd(Self, ASeg);
    case Thread.Player of
      plSegment: ASeg.Stop;
      plMIDPlay:
        if moLoop in Options then
        begin Stop; Play;
        end;
      plPlayList:
        begin PlayList.PlayNext;
        end;


    end;
  end;

end;




////////////  PLAY LIST

destructor TPListItem.Destroy;
 begin
  Seg.Stop;

  Seg.Unload;
  Seg.Seg:=nil;
  inherited;
 end;


constructor TMIDPlayList.Create(Ply: TMIDPlay);
begin
  inherited Create;
  MIDPlay := Ply;
  FList := TObjectList.Create;
  //FMIDList := TMIDList.Create(Ply);
  FOptions := [poContinuous];
  FStrList := TStringList.Create;
  ItemIdx := -1;
end;

destructor TMIDPlayList.Destroy;
begin
  FStrList.Free;
  //FMIDList.Free;
  FList.Free;
  StoreList := nil;
  inherited Destroy;
end;


procedure TMIDPlayList.Play;
begin
  if ASeg <> nil then
  begin
    ASeg.Play;
    MIDPlay.Thread.Start(plPlayList);
  end;

end;

procedure TMIDPlayList.PlayNext;
var
  next, i: long;
  ex: boolean;
begin
  ex := false;
  if AItem <> nil then AItem.Played := true;
   for i := 0 to List.Count - 1 do
    if not TPlistItem(List[i]).Played then ex := true;

  if not ex then
  begin Restart(poContinuous in Options);Exit;
  end;

  repeat
    if poShuffle in Options then
      itemidx := Random(List.Count)
    else
      inc(itemidx, 1);
    if Itemidx > List.Count - 1 then
    begin Restart(poContinuous in Options); Exit;
    end;
  until not TPlistItem(List[itemidx]).Played;
  if ASeg <> nil then ASeg.Stop;
  ASeg := TPlistItem(List[itemidx]).Seg;
  AItem := TPlistItem(List[itemidx]);
  Play;

end;

procedure TMIDPlayList.PlayItem(idx: long);
begin
  if (idx > List.Count - 1) or (idx < 0) then Exit;
  ItemIdx := idx;
  ASeg := TPlistItem(List[itemidx]).Seg;
  Play;

end;

procedure TMIDPlayList.Pause;
begin
  if Aseg <> nil then ASeg.Pause;
end;

procedure TMIDPlayList.Restart(Ply: boolean);
var
  i: long;
begin
  if ASeg <> nil then ASeg.Stop;
  if AItem <> nil then AItem := nil;
  for i := 0 to List.Count - 1 do
    TPlistItem(List[i]).Played := false;
  itemidx := -1;
  if Ply then PlayNext;
end;

procedure TMIDPlayList.LoadFromList(Pl: TStrings; Ply: boolean);
begin
  LoadFromCustomList(Pl, StoreList, Ply);
end;

procedure TMIDPlayList.LoadFromCustomList(Pl: TStrings; List: TMIDList; Ply: boolean);
var
  i: long;
  s: string;
  Seg: TSegment;
  Item: TPListitem;
  MI:TMIDCollectionItem;
  ii:long;
begin
  if (PL = nil) or (List = nil) then Exit;
  StrList.Clear;
  FList.Clear;

  for i := 0 to Pl.Count - 1 do
  begin
    if List.Items.Find(PL[i]) <> nil then
     MI:=List.Items.Find(Pl[i])
    else
      Continue;

    s:=MI.Name;
    Seg := TSegment.Create(MIDPlay);
    Item := TPListItem.Create;
    Item.Name := S;
    Item.Seg := Seg;
    Seg.LoadFromMemory(MI.MID.MS.memory,MI.MID.MS.size);
    MIDPlay.SegList.Add(Seg);
    FList.Add(Item);
    StrList.Add(s);

  end;
  if Ply then Restart(true);

end;

procedure TMIDPlayList.LoadFromFiles(Pl: TStrings; Ply: boolean);
var
  i: long;
  s: string;
  Seg: TSegment;
  ITem: TPListItem;
begin
  if Pl = nil then Exit;
  StrList.Clear;
  for i:=0 to List.Count-1 do
   MIDPlay.SegList.Remove(TPListItem(List[i]).Seg);

  List.Clear;

  for i := 0 to Pl.Count - 1 do
  begin
    if Fileexists(Pl[i]) then
      s := Pl[i]
    else if Fileexists(ExeDir + Pl[i]) then
      s := Exedir + Pl[i]
    else
      Continue;
    Seg := TSegment.Create(MIDPlay);
    Item := TPListItem.Create;
    Item.Name := S;
    Item.Seg := Seg;
    Seg.LoadFromFile(s);
    MIDPlay.SegList.Add(Seg);
    List.Add(Item);
    StrList.Add(s);

  end;
  if Ply then Restart(true);

end;

procedure TMIDPlayList.LoadFromItem(Item: TTextCollectionItem; From: TLoadFrom; Ply: boolean);
begin
  if Item = nil then Exit;
  case From of
    lfFiles: LoadFromFiles(Item.Text.Lines, Ply);
    lfList: LoadFromList(Item.Text.Lines, Ply);
  end;
end;
 ////////////  SEGMENT LIST

function TSEgList.Add(ASeg: TSegment): TSegment;
begin
  inherited Add(ASeg);
  Result := ASeg;
end;


constructor TSEgList.Create;
begin
  inherited Create;
  FOwnsObjects := True;
end;

constructor TSEgList.Create(AOwnsObjects: Boolean);
begin
  inherited Create;
  FOwnsObjects := AOwnsObjects;
end;


function TSEgList.GetItem(Index: Integer): TSegment;
begin
  Result := inherited Items[Index];
end;

function TSEgList.IndexOf(ASeg: TSegment): Integer;
begin
  Result := inherited IndexOf(ASeg);
end;

procedure TSEgList.Insert(Index: Integer; ASeg: TSegment);
begin
  inherited Insert(Index, ASeg);
end;

procedure TSEgList.Notify(Ptr: Pointer; Action: TListNotification);
begin
  if OwnsObjects then
    if Action = lnDeleted then
      TSegment(Ptr).Free;
  inherited Notify(Ptr, Action);
end;

function TSEgList.Remove(ASeg: TSegment): Integer;
begin
  Result := inherited Remove(ASeg);
  ASeg.Unload;
  ASeg.Seg:=nil;
  ASeg:=nil;
end;

procedure TSEgList.SetItem(Index: Integer; ASeg: TSegment);
begin
  inherited Items[Index] := ASeg;
end;


/////////////  TPLAYTHREAD METHODS ////////////////

constructor TPlayThread.Create(Ply: TMIDPlay; PL: TMIDPlayList);
begin
  inherited Create(true);
  Priority := tpIdle;
  Play := Ply;
  List := PL;
end;

destructor TPlayThread.Destroy;
begin
  Suspended := true;
  Play := nil;
  List := nil;
  inherited Destroy;
end;

procedure TPlayThread.Start(Mode: TPlayer);
begin
  Player := Mode;
  Suspended:=false;

end;

procedure TPlayThread.Execute;
begin
  while not Terminated do
  begin
    Synchronize(Play.Update);
  end;
end;

/////////////////////////// TPORT //////////////////////
constructor TPort.Create(Ply: TMIDPlay);
begin
  inherited Create;
  FMIDPlay := Ply;
  FIndex:=1;
  FName:='Microsoft Synthesizer';
end;

destructor TPort.Destroy;
begin
  FPort := nil;
  inherited Destroy;
end;


function TPort.GetPortCount: long;
var i,Idx:long;
    Caps:TDMUS_PORTCAPS;
    exitus:boolean;
begin
idx:=0;
i:=1;
exitus:=false;
repeat
 ZeroMemory(@Caps,SizeOf(Caps));
 Caps.dwSize:=SizeOf(Caps);
 if FMIDPlay.FMusic.EnumPort(i,Caps)=S_OK then Inc(idx)
 else Exitus:=true;
Inc(i);
until Exitus;
Result:=idx;
end;

procedure TPort.SetPortIdx(idx: long);
var Caps:TDMUS_PORTCAPS;
    Params:TDMUS_PORTPARAMS;
    pl:boolean;
begin
if not FMidPlay.Ready then Exit;
pl:=FMIDPlay.PlayingAll;
if pl then FMidPlay.ASeg.Pause;
Caps:=GetPortCaps(idx);
ZeroMemory(@Params,SizeOf(Params));
Params.dwSize:=SizeOf(Params);
Params.dwChannelGroups :=1;
Params.dwValidParams :=DMUS_PORTPARAMS_CHANNELGROUPS;
FMIDPlay.Fperf.RemovePort(FPort);
FPort:=nil;
FMIDPlay.FMusic.CreatePort(Caps.guidPort,Params,FPort,nil);
FMIDPlay.FPerf.AddPort(FPort);
FMidPlay.FPerf.AssignPChannelBlock(0,FPort,1);

FPort.Activate(true);
FMidPlay.ASeg.Seg.setParam(GUID_Download, $FFFFFFFF, 0, 0, Pointer(fMIDPlay.FPerf));
if Pl then FMIDPlay.ASeg.Play;
end;

procedure TPort.SetPortName(Name: string);
begin
 SetPortIdx(GetNameindex(name));
end;

function TPort.GetNameIndex(PName:string):long;
var i:long;
begin
 for i:=1 to PortCount do
  if CompStr(PName,GetPortName(i)) then
   begin
   Result:=i;
   Break;
   end;
end;

function TPort.GetPortName(idx: long): string;
var Caps:TDMUS_PORTCAPS;
begin
Caps:=Getportcaps(idx);
Result:=Caps.wszDescription;


end;

function TPort.GetPortCaps(idx: long): TDMUS_PORTCAPS;
var Caps:TDMUS_PORTCAPS;
begin
ZeroMemory(@Caps,SizeOf(caps));
Caps.dwSize:=Sizeof(Caps);
if not FMiDPlay.Ready then Exit;
FMiDPlay.FMusic.EnumPort(idx,Caps);
Result:=Caps;

end;






procedure Register;
begin
  RegisterComponents('Vampyre', [TMIDPlay]);
end;

end.

