{
  Drake Rider

  (C) 2002 Vampyre SoftWorks
  Created by :  Marek Mauder  
}	


unit Maps;

interface

uses Windows,Classes,SysUtils,gameTypes,WinFunc,
EngineObj,EngEquip,MainGame,Objects;


const

    MAX_WORLD_X=50000;
    MAX_WORLD_Y=600;


type


   TMap=class
       public
        Name:string;
        ShortName:string;
        MaxWorldX:long;
        MaxWorldY:long;
        Ground:long;
        Sky:long;

        ThemeMus:string;

        Objects:TBaseObjList;
        Infos:TBaseInfoList;

        constructor Create;
        destructor Destroy;override;
   
        function AddObj(Obj:TBaseObj):TBaseObj;
        function AddInfo(Inf:TBaseInfo):TBaseInfo;

        procedure SaveToFile(F:string);
        procedure SaveToStream(S:TStream);
        procedure LoadFromStream(S:TStream);
        procedure LoadFromFile(F:string);
        
   
   

       end;

type

   TMap_HDR=record
             ID:TChar4;
             Sp1:Dword;
             Info:TChar20;
             VerHIWORD:word;
             VerLOWORD:word;
             Size:dword;
             Sp2:Dword;
            end;


            
   
const 

   Map_EXT='.drm';
   Map_ID='DRMP';
   Map_Info='DRAKE RIDER MAP FILE';
   Map_ObjSec='Map Objects:';
   
   Map_HDR:TMap_Hdr=(ID:Map_ID;
                   Sp1:Byte('@') or (5 shl 8) or (5 shl 16) or (Byte('@') shl 24 );
                   Info:Map_Info;VERHIWORD:VerHi;VERLOWORD:VerLo;Size:0;
                   Sp2:Byte('@') or (5 shl 8) or (5 shl 16) or (Byte('@') shl 24 )
                   
   );

   

implementation

////////////////////////  TMAP

constructor TMap.Create;
begin
 inherited Create;
 Objects:=TBaseObjList.Create;
 Infos:=TBaseInfoList.Create;
 MaxWorldX:=5000;
 MaxWorldY:=MAX_WORLD_Y;
 Ground:=20;
 Sky:=20;

 Name:='Meanwhile Unknown Map';
 ShortName:='untitled.drm';

end;

destructor TMap.Destroy;
begin
 Infos.Free;
 Objects.Free;
 inherited;
end;

function TMap.AddObj(Obj: TbaseObj): TBaseObj;
var
  L, H, I, C: Integer;
begin
  L := 0;
  H := Objects.Count - 1;
  while L <= H do
  begin
    I := (L + H) div 2;
    C := Objects[I].Z - Obj.Z;
    if C < 0 then
      L := I + 1
    else
      H := I - 1;
  end;
  Objects.Insert(L, Obj);
  Result := Obj;
end;


function TMap.AddInfo(Inf: TbaseInfo): TBaseInfo;
var
  L, H, I, C: Integer;
begin
  L := 0;
  H := Infos.Count - 1;
  while L <= H do
  begin
    I := (L + H) div 2;
    C := Infos[I].Z - Inf.Z;
    if C < 0 then
      L := I + 1
    else
      H := I - 1;
  end;
  Infos.Insert(L, Inf);
  Result := Inf;
end;



procedure TMap.SaveToFile(F:string);
var MS:TMS;
begin
 MS:=TMS.Create;
 Shortname:=ExtractFileName(F);
 SaveToStream(MS);
 MS.SaveToFile(F);
 MS.Free;
end;

procedure TMap.SaveToStream(S:TStream);
var B:boolean;
  i:long;
begin
S.WriteBuffer(Map_Hdr,SizeOf(Map_Hdr));

WriteStr(S,Name);
S.WriteBuffer(B,sizeOf(B));

WriteStr(S,ShortName);
S.WriteBuffer(B,sizeOf(B));

S.WriteBuffer(MaxWorldX,SizeOf(long));
S.WriteBuffer(MaxWorldY,SizeOf(long));
S.WriteBuffer(Ground,SizeOf(long));
S.WriteBuffer(Sky,SizeOf(long));

WriteStr(S,ThemeMus);

WriteStr(S,Map_ObjSec);

S.WriteBuffer(Infos.Count,SizeOf(Infos.Count));

for i:=0 to Infos.Count-1 do
 begin
 WriteStr(S,Infos[i].Classname);
 if ObjMgr.Find(Infos[i].Name)<>nil then
 begin
  WriteStr(S,Infos[i].Name);
  S.WriteBuffer(Infos[i].X,SizeOf(long));
  S.WriteBuffer(Infos[i].Y,SizeOf(long));

 end
 else
 Infos[i].Save(S);

 end;



end;

procedure TMap.LoadFromStream(S:TStream);
var hd:TMap_Hdr;
    cnt,i:long;
    pos:long;
    B:boolean;
    nm:string;
    ix,iy:long;
    st:string;
    O:TBaseInfo;
begin
S.ReadBuffer(Hd,SizeOf(Hd));
if not Compstr(Hd.ID,MAp_ID) then begin MsgBox('Not valid DR Map file!');exit;end;

//if (Hd.VerHIWORD<>verHi)or(Hd.VerLOWORD<>verLo) then begin MsgBox('Map version mismatch!May mot work!');end;

Objects.Clear;
Infos.Clear;

Name:=ReadStr(S);
S.ReadBuffer(B,sizeOf(B));
ShortName:=ReadStr(S);
S.ReadBuffer(B,sizeOf(B));

S.ReadBuffer(MaxWorldX,SizeOf(long));
S.ReadBuffer(MaxWorldY,SizeOf(long));
S.ReadBuffer(Ground,SizeOf(long));
S.ReadBuffer(Sky,SizeOf(long));

ThemeMus:=ReadStr(S);
st:=ReadStr(S);
S.ReadBuffer(Cnt,SizeOf(Cnt));

for i:=0 to cnt-1 do
 begin
  st:=ReadStr(S);
  O:=TEngObjInfo(FindClass(st)).Create;
  pos:=S.Position;
  nm:=ReadStr(S);
  if ObjMgr.Find(nm)<>nil then
  begin
   O.Free;
   O:=ObjMgr.Find(nm).Clone;
   S.ReadBuffer(O.X,SizeOf(long));
   S.ReadBuffer(O.Y,SizeOf(long));

  end
  else
  begin
  S.Position:=pos;
  O.Load(S);
  end;
  Infos.Add(O);

 end;

end;

procedure TMap.LoadFromFile(F:string);
var MS:TMS;
begin
 MS:=TMS.Create;
 MS.LoadFromFile(F);
 LoadFromStream(MS);
 MS.Free;
end;



end.
