{
  Drake Rider

  (C) 2002 Vampyre SoftWorks
  Created by :  Marek Mauder  
}	


unit Sound;

interface

uses Windows, Classes, gameTypes, SysUtils, WinFunc, MainGame,
  Bass, Errors, ContNRS;

type


  TBassObj = class
  public
    Handle: DWord;
      Name: string;
    F: TLoadFile;

    Tag: long;

    constructor Create(AName: string; AInit: boolean = true); virtual;
    destructor Destroy; override;

    procedure Init; virtual;
    procedure Play; virtual;
    procedure Stop;virtual;

  end;

  TBoClass = class of TBassObj;

  TSample = class(TBassObj)
  public

    constructor Create(AName: string; AInit: boolean = true); override;
    destructor Destroy; override;

    procedure Init; override;
    procedure Play; override;
    procedure Stop; override;

           //procedure PlayEx;

  end;

  TBStream = class(TBassObj)
  public
    constructor Create(AName: string;AInit: boolean = true); override;
    destructor Destroy; override;

    procedure Init; override;
    procedure Play(Looped:boolean=true);virtual;
    procedure Stop; override;

           //procedure PlayEx;

  end;


  
  TSoundMgr = class
  public
    Objects: TObjectList;

    Handle: Hwnd;
    Freq: long;

    Started: boolean;
    constructor Create(AHandle: Hwnd; AFreq: long = 44100; AInit: boolean = true);
    destructor Destroy; override;

    procedure Init;

    function GetSnd(Name: string; Cls: TBoClass = nil; Tag: long = 0): TBassObj;
    function IsSnd(Name: string): boolean;
    function Find(Name: string): TBassObj;
    function AddSnd(SX: TBassObj; Tag: long = 0): TBassObj;
    procedure RemoveSnd(Name: string); overload;
    procedure RemoveSnd(snd: TBassObj); overload;
    function RemoveTag(Tag: long): long;
    function CountTag(Tag: long): long;


    function GetCPUUsage: long;
    procedure SetVol(vol: long);
    procedure Start;
    procedure Stop;

  end;

var
  SoundMgr: TSoundMgr;

implementation

///////////////////// TBASSOBJ

constructor TBassObj.Create(AName: string; AInit: boolean);
begin
  inherited Create;
  Handle := 0;
  Name := AName;

  if AInit then Init;
end;

destructor TBassObj.Destroy;
begin

  inherited;
end;

procedure TbassObj.Init();
begin


end;

procedure TbassObj.Play();
begin

end;

procedure TbassObj.Stop();
begin

end;


/////////////////////////// TSAMPLE

constructor TSample.Create(AName: string; AInit: boolean);
begin
  inherited;

end;

destructor TSample.Destroy;
begin
  if Handle > 0 then BASS_SampleFree(Handle);
  FreeFile(F);
  inherited;
end;

procedure TSample.Init;
begin
  inherited;
  if Handle > 0 then BASS_SampleFree(handle);

  FreeFile(F);
  F := LoadFile(name);
  Handle := BASS_SampleLoad(true, F.Bytes, 0, F.Size, 2, 0);
  if handle = 0 then ShowError(ERR_SAMPLELOAD);

end;

procedure TSample.Play();
begin
  inherited;
  if Handle > 0 then BASS_SamplePlay(Handle);

end;

procedure Tsample.Stop();
begin
 if Handle>0 then BASS_SampleStop(Handle);

end;


/////////////////////////// TBSTREAM

constructor TBStream.Create(AName: string; AInit: boolean);
begin
  inherited Create(AName,Ainit); 
end;

destructor TBStream.Destroy;
begin
  if Handle > 0 then BASS_StreamFree(Handle);
  FreeFile(F);
  inherited;
end;

procedure TBStream.Init;
begin
  inherited;
  if Handle > 0 then BASS_StreamFree(handle);

  FreeFile(F);
  F := LoadFile(name);
  Handle:=Bass_StreamCreateFile(true, F.Bytes, 0, F.Size, 0);
  if handle = 0 then ShowError(ERR_SAMPLELOAD);

  
end;

procedure TBStream.Play(Looped:boolean);
var flag:long;
begin
  inherited Play;
  flag:=0;
  if Looped then flag:=BASS_SAMPLE_LOOP;
  
  if Handle > 0 then BASS_StreamPlay(Handle,true,flag);

end;

procedure TBStream.Stop();
begin
 if Handle>0 then BASS_StreamFree(Handle);
 Init;
end;






///////////////////////   TSOUNDMGR

constructor TSoundMgr.Create(AHandle: Hwnd; AFreq: long; AInit: boolean);
begin
  inherited Create;
  Freq := AFreq;
  Handle := AHandle;
  Objects := TObjectList.Create;
  if Ainit then Init;
end;

destructor TSoundMgr.Destroy;
begin
  Stop;

  Objects.Free;
  BASS_Free;

  inherited;
end;

procedure TSoundMgr.Init();
begin

  BASS_Init(-1, freq, 0, Handle);
  Start();

end;




function TSoundMgr.GetSnd(Name: string; Cls: TBOClass; Tag: long): TBassObj;
var
  sX: TBassObj;
begin
  Result := nil;
  if Name='' then Exit;
  if IsSnd(Name) then
    Result := Find(name)
  else
  begin
    if Cls = nil then
      sX := TSample.Create(Name)
    else
      sX := Cls.Create(Name);
    sX.Tag := Tag;
    Objects.Add(sX);
    Result := sX;
  end;


end;

function TSoundMgr.Find(Name: string): TBassObj;
var
  i: long;
begin
  Result := nil;
  if not IsSnd(Name) then Exit;

  for i := 0 to Objects.Count - 1 do
    if CompStr(Name, TBassObj(Objects[i]).name)
      then
    begin
      Result := TBassObj(Objects[i]);
      Break;
    end;



end;

function TSoundMgr.IsSnd(Name: string): boolean;
var
  i: long;
begin
  Result := false;
  for i := 0 to Objects.Count - 1 do
    if CompStr(Name, TBassObj(Objects[i]).name)
      then
    begin
      Result := true;
      Break;
    end;


end;

function TSoundMgr.AddSnd(sX: TBassObj; Tag: long): TBassObj;
begin

  sX.Tag := Tag;
  if IsSnd(sX.Name) then RemoveSnd(sx.Name);
  Objects.Add(sX);
  Result := sX;
end;

procedure TSoundMgr.RemoveSnd(Name: string);
var
  sx: TBassObj;
begin
  if not IsSnd(Name) then Exit;
  sx := Find(name);
  Objects.Remove(sX);


end;

procedure TSoundMgr.RemoveSnd(snd: TBassObj);
begin
  Objects.Remove(snd);
end;

function TSoundMgr.GetCPUUsage: long;
begin
  Result := Round(BASS_GetCPU * 100);
end;

procedure TSoundMgr.SetVol(vol: long);
begin
  if vol = 1 then vol := 0;
  BASS_SetGlobalVolumes(vol * 20, vol * 20, vol * 20);


end;

procedure TSoundMgr.Start();
begin
  Started := true;
  BASS_Start;

end;

procedure TSoundMgr.Stop();
begin
  Started := false;
  BASS_Stop;
end;



function TSoundMgr.RemoveTag(Tag: long): long;
var
  i, ii: long;
begin
  Result := 0;
  ii := Objects.Count - 1;
  for i := 0 to ii do
    if i >= Objects.Count then
      break
    else if TBassObj(Objects[i]).Tag = tag then
    begin
      Objects.Remove(Objects[i]);
      Inc(Result);
    end;

end;

function TSoundMgr.CountTag(Tag: long): long;
var
  i: long;
begin
  Result := 0;
  for i := 0 to Objects.Count - 1 do
    if TBassObj(Objects[i]).tag = tag then Inc(Result);

end;





end.

