unit VTextList;
{
*********************************************************************
*********************************************************************
Component : TextList

Vampyre SoftWorks
http:\www.vampyre.cz?????

Final Version 25.1.2001

Developer : Marek Mauder
            pentar@seznam.cz

*********************************************************************
*********************************************************************
}



interface

uses
  Windows, Messages, SysUtils, Classes,
  Controls, Hash;

type
  TTextCollection = class;

  TStr = class(TPersistent)
  private
  protected
    procedure ReadData(Stream: TStream); virtual;
    procedure WriteData(Stream: TStream); virtual;
    procedure DefineProperties(Filer: TFiler); override;
  public
    Lines: TStringList;
    constructor Create(AOwner: TObject);
    destructor Destroy; override;
    procedure SaveToStream(STream: TStream);
    procedure LoadFromStream(STream: TStream);
  end;



  TTextCollectionItem = class(THashCollectionItem)
  private
    FStr: TStr;
    function GetTextCollection: TTextCollection;

  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    property TextCollection: TTextCollection read GetTextCollection;
    procedure Assign(Source: TPersistent);override;
  published
    property Text: TStr read FStr write FStr;
  end;



  TTextCollection = class(THashCollection)
  private

    FOwner: TPersistent;
    function GetItem(Index: Integer): TTextCollectionItem;

  protected
    function GetOwner: TPersistent; override;
  public
    constructor Create(AOwner: TPersistent);
    function Find(const Name: string): TTextCollectionItem;
    procedure LoadFromFile(const FileName: string);
    procedure LoadFromStream(Stream: TStream);
    procedure SaveToFile(const FileName: string);
    procedure SaveToStream(Stream: TStream);
    property Items[Index: Integer]: TTextCollectionItem read GetItem; default;
  end;



  TCustomTextList = class(TComponent)
  private
    FItems: TTextCollection;
    procedure SetItems(Value: TTextCollection);
  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Items: TTextCollection read FItems write SetItems;
  end;


  TTextList = class(TCustomTextList)
  published
    property Items;
  end;

{  TTextProperty = class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetValue: string; override;
    function GetAttributes: TPropertyAttributes; override;
  end;
 }


//procedure Register;

implementation
//uses VTFormE;

constructor TStr.Create(AOwner: TObject);
begin
  inherited Create;
  Lines := TStringList.Create;
end;

destructor TStr.Destroy;
begin
  Lines.Free;
  inherited Destroy;
end;

procedure TStr.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);
  Filer.DefineBinaryProperty('Textdata', ReadData, WriteData, true);
end;

procedure TStr.ReadData(Stream: TStream);
begin
  LoadFromStream(Stream);
end;

procedure TStr.WriteData(Stream: TStream);
begin
  SaveToStream(Stream);
end;

procedure TStr.SaveToStream(Stream: TStream);
begin
  Lines.SaveToStream(Stream);
end;

procedure TStr.LoadFromStream(Stream: TStream);
begin
  Lines.LoadFromStream(Stream);
end;

constructor TTextCollectionItem.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  FStr := TStr.Create(Self);
end;

destructor TTextCollectionItem.Destroy;
begin
  FStr.Free;
  inherited Destroy;
end;

procedure TTextCollectionItem.Assign(Source: TPersistent);
begin
  if Source is TTextCollectionItem then
  begin
    inherited Assign(Source);
    Text.Lines.Assign(TTextCollectionItem(Source).Text.Lines);
  end else
    inherited Assign(Source);
end;



function TTextCollectionItem.GetTextCollection: TTextCollection;
begin
  Result := Collection as TTextCollection;
end;


constructor TTextcollection.Create(AOwner: TPersistent);
begin
  inherited Create(TTextCollectionItem);
  FOwner := AOwner;
end;

function TTextCollection.GetItem(Index: Integer): TTextCollectionItem;
begin
  Result := TTextCollectionItem(inherited Items[Index]);
end;

function TTextCollection.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

function TTextCollection.Find(const Name: string): TTextCollectionItem;
var
  i: Integer;
begin
  Result:=nil;
  i := IndexOf(Name);
  if i>-1 then 
  Result := Items[i];
end;


type
  TTextCollectionComponent = class(TComponent)
  private
    FList: TTextCollection;
  published
    property List: TTextCollection read FList write FList;
  end;

procedure TTextCollection.LoadFromFile(const FileName: string);
var
  Stream: TFileStream;
begin
  Stream := TFileStream.Create(FileName, fmOpenRead or fmShareDenyWrite);
  try
    LoadFromStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure TTextCollection.LoadFromStream(Stream: TStream);
var
  Component: TTextCollectionComponent;
begin
  Clear;
  Component := TTextCollectionComponent.Create(nil);
  try
    Component.FList := Self;
    Stream.ReadComponent(Component);

  finally
    Component.Free;
  end;
end;

procedure TTextCollection.SaveToFile(const FileName: string);
var
  Stream: TFileStream;
begin
  Stream := TFileStream.Create(FileName, fmCreate);
  try
    SaveToStream(Stream);
  finally
    Stream.Free;
  end;
end;

procedure TTextCollection.SaveToStream(Stream: TStream);
var
  Component: TTextCollectionComponent;
begin
  Component := TTextCollectionComponent.Create(nil);
  try
    Component.FList := Self;
    Stream.WriteComponent(Component);
  finally
    Component.Free;
  end;
end;

constructor TCustomTextList.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FItems := TTextCollection.Create(Self);
end;

destructor TCustomTextList.Destroy;
begin
  FItems.Free;
  inherited Destroy;
end;

procedure TCustomTextList.SetItems(Value: TTextCollection);
begin
  FItems.Assign(Value);
end;

{procedure TTextProperty.Edit;
var
  Form: TTextEditForm;
begin
  Form := TTextEditForm.Create(nil);
  try
    Form.Str := TStr(GetOrdValue);
    Form.ShowModal;
    if Form.Tag <> 0 then
    begin
      SetOrdValue(Integer(Form.Str));
      Designer.Modified;
    end;
  finally
    Form.Free;
  end;
end;

function TTextProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

function TTextProperty.GetValue: string;
begin
  if TStr(GetOrdValue).Lines.Count < 1 then
    Result := '(NONE)'
  else
    Result := Format('(%s)', [TObject(GetOrdValue).ClassName]);
end;
 }


//procedure Register;
{begin
//  RegisterComponents('Vampyre', [TTextList]);
end;
 }
end.

