unit WinFunc;
/////////////////////////////////////////////////////////////
/////   MISCELLANEOUS FUNCTIONS
////////////////////////////////////////////////////////////
/////   VAMPYRE SOFTWORKS
////////////////////////////////////////////////////////////
/////
////////////////////////////////////////////////////////////

interface
uses Classes, Windows,Forms,  SysUtils;


function CompStr(S1, S2: string): boolean;
function CompStrArray(S1: string; Arr: array of string): boolean;
function ExeDir: string;
function StrInList(S: string; List: TStrings): boolean;
procedure WriteStr(Stream: TStream; S: string);
function ReadStr(Stream: TStream): string;
function ForceDirectories(Dir: string): Boolean;
function DirectoryExists(const Name: string): Boolean;
function CurPos: TPoint;
function MsgBox(S: string): longint;
procedure ExitApp;
function Max(A, B: Integer): Integer;
function Min(A, B: Integer): Integer;
procedure Swap(var a, b: Longint);
function ClipRect(var DestRect: TRect; const DestRect2: TRect): Boolean;
function ClipRect2(var DestRect, SrcRect: TRect; const DestRect2, SrcRect2: TRect): Boolean;
function Sgn(a: Longint): Longint;
function DegToRad(Degrees: Extended): Extended;
function SubRect(R1, R2: TRect): TRect;


implementation

function DegToRad(Degrees: Extended): Extended;
begin
  Result := Degrees * (PI / 180);
end;


function Sgn(a: Longint): Longint;
begin
  if a > 0 then
    result := +1
  else if a < 0 then
    result := -1
  else
    result := 0;
end;



function ClipRect(var DestRect: TRect; const DestRect2: TRect): Boolean;
begin
  with DestRect do
  begin
    Left := Max(Left, DestRect2.Left);
    Right := Min(Right, DestRect2.Right);
    Top := Max(Top, DestRect2.Top);
    Bottom := Min(Bottom, DestRect2.Bottom);

    Result := (Left < Right) and (Top < Bottom);
  end;
end;


function ClipRect2(var DestRect, SrcRect: TRect; const DestRect2, SrcRect2: TRect): Boolean;
begin
  if DestRect.Left < DestRect2.Left then
  begin
    SrcRect.Left := SrcRect.Left + (DestRect2.Left - DestRect.Left);
    DestRect.Left := DestRect2.Left;
  end;

  if DestRect.Top < DestRect2.Top then
  begin
    SrcRect.Top := SrcRect.Top + (DestRect2.Top - DestRect.Top);
    DestRect.Top := DestRect2.Top;
  end;

  if SrcRect.Left < SrcRect2.Left then
  begin
    DestRect.Left := DestRect.Left + (SrcRect2.Left - SrcRect.Left);
    SrcRect.Left := SrcRect2.Left;
  end;

  if SrcRect.Top < SrcRect2.Top then
  begin
    DestRect.Top := DestRect.Top + (SrcRect2.Top - SrcRect.Top);
    SrcRect.Top := SrcRect2.Top;
  end;

  if DestRect.Right > DestRect2.Right then
  begin
    SrcRect.Right := SrcRect.Right - (DestRect.Right - DestRect2.Right);
    DestRect.Right := DestRect2.Right;
  end;

  if DestRect.Bottom > DestRect2.Bottom then
  begin
    SrcRect.Bottom := SrcRect.Bottom - (DestRect.Bottom - DestRect2.Bottom);
    DestRect.Bottom := DestRect2.Bottom;
  end;

  if SrcRect.Right > SrcRect2.Right then
  begin
    DestRect.Right := DestRect.Right - (SrcRect.Right - SrcRect2.Right);
    SrcRect.Right := SrcRect2.Right;
  end;

  if SrcRect.Bottom > SrcRect2.Bottom then
  begin
    DestRect.Bottom := DestRect.Bottom - (SrcRect.Bottom - SrcRect2.Bottom);
    SrcRect.Bottom := SrcRect2.Bottom;
  end;

  Result := (DestRect.Left < DestRect.Right) and (DestRect.Top < DestRect.Bottom) and
    (SrcRect.Left < SrcRect.Right) and (SrcRect.Top < SrcRect.Bottom);
end;



procedure Swap(var a, b: Longint);
var
  c: Longint;
begin
  c := a;
  a := b;
  b := c;
end;

function Max(A, B: Integer): Integer;
begin
  if A > B then
    Result := A
  else
    Result := B;
end;


function Min(A, B: Integer): Integer;
begin
  if A < B then
    Result := A
  else
    Result := B;
end;


function DirectoryExists(const Name: string): Boolean;
var
  Code: Integer;
begin
  Code := GetFileAttributes(PChar(Name));
  Result := (Code <> -1) and (FILE_ATTRIBUTE_DIRECTORY and Code <> 0);
end;



function ForceDirectories(Dir: string): Boolean;
begin
  Result := True;
  if Length(Dir) = 0 then
    Exit;
  Dir := ExcludeTrailingBackslash(Dir);
  if (Length(Dir) < 3) or DirectoryExists(Dir)
    or (ExtractFilePath(Dir) = Dir) then Exit; // avoid 'xyz:\' problem.
  Result := ForceDirectories(ExtractFilePath(Dir)) and CreateDir(Dir);
end;





function ExeDir: string;
begin
  Result := ExtractFileDir(ParamStr(0)) + '\';

end;

function CompStr(S1, S2: string): boolean;
begin
  if AnsiCompareText(S1, S2) = 0 then
    Result := true
  else
    Result := false;
end;

function CompStrArray(S1: string; Arr: array of string): boolean;
var
  i: longint;
begin
  Result := false;
  for i := 0 to High(Arr) do
    if CompStr(S1, Arr[i]) then Result := true;

end;



function StrInList(S: string; List: TStrings): boolean;
var
  i: longint;
begin
  Result := false;
  for i := 0 to List.Count - 1 do
    if CompStr(S, List.Strings[i]) then Result := true;

end;

procedure WriteStr(Stream: TStream; S: string);
var
  Writer: TWriter;
begin
  Writer := TWriter.Create(Stream, 8192);
  Writer.WriteString(S);
  Writer.Free;
end;

function ReadStr(Stream: TStream): string;
var
  Reader: TReader;
begin
  Reader := TReader.Create(Stream, 8192);
  Result := Reader.ReadString;
  Reader.Free;
end;


function CurPos: TPoint;
var
  P: TPoint;
begin
  GetCursorPos(P);
  Result := P;
end;


function MsgBox(S: string): longint;
begin
  Result := Application.MessageBox(Pchar(S), 'Message', 0);
end;


procedure ExitApp;
begin
  Application.Terminate;
end;

function SubRect(R1, R2: TRect): TRect;
begin
  Result := Rect(r1.left - r2.left, r1.top - r2.top, r1.right - r2.left, r1.bottom - r2.top);

end;


end.

